@@ -1114,6 +1114,12 @@ ttsspd(speed) {
 	    case 2400: s = B2400; break;
 	    case 4800: s = B4800; break;
 	    case 9600: s = B9600; break;
+#ifdef MINIX
+	    case 19200: s = B19200; break;
+	    case 38400: s = B38400; break;
+	    case 57600: s = B57600; break;
+	    case 115200: s = B115200; break;
+#endif
 #ifdef PLEXUS
 	    case 19200: s = EXTA; break;
 #endif
@@ -1744,6 +1750,9 @@ ttsndb() {
     }
     return(x);
 #else
+#ifdef MINIX
+    tcsendbreak(ttyfd, 0);
+#else
 #ifdef	V7
     genbrk(ttyfd);			/* Simulate a BREAK */
     return(x);
@@ -1752,6 +1761,7 @@ ttsndb() {
 #endif
 #endif
 #endif
+#endif
 }
 
 /*  M S L E E P  --  Millisecond version of sleep().  */
