@@ -5,8 +5,6 @@
  * The entry points into this file are
  *   do_dup:	  perform the DUP system call
  *   do_fcntl:	  perform the FCNTL system call
- *   lock_op:	  handle the setting of locks
- *   lock_revive: revive processes when a lock is released
  *   do_sync:	  perform the SYNC system call
  *   do_fork:	  adjust the tables after MM has performed a FORK system call
  *   do_exec:	  handle files with FD_CLOEXEC on after MM has done an EXEC
@@ -25,10 +23,9 @@
 #include "file.h"
 #include "fproc.h"
 #include "inode.h"
+#include "dev.h"
 #include "param.h"
 
-FORWARD _PROTOTYPE( int lock_op, (struct filp *f, int req)		);
-
 
 /*===========================================================================*
  *				do_dup					     *
@@ -73,7 +70,7 @@ PUBLIC int do_dup()
  *===========================================================================*/
 PUBLIC int do_fcntl()
 {
-/* Perform the fcntl(fd, request, addr) system call. */
+/* Perform the fcntl(fd, request, ...) system call. */
 
   register struct filp *f;
   int new_fd, r, fl;
@@ -128,179 +125,6 @@ PUBLIC int do_fcntl()
 }
 
 
-/*===========================================================================*
- *				lock_op					     *
- *===========================================================================*/
-PRIVATE int lock_op(f, req)
-struct filp *f;
-int req;			/* either F_SETLK or F_SETLKW */
-{
-/* Perform the advisory locking required by POSIX. */
-
-  int r, ltype, mo, i, conflict = 0, unlocking = 0;
-  off_t first, last;
-  struct flock flock;
-
-  char *fs_addr;
-  vir_bytes user_flock, fs_flock, flock_size;
-  struct file_lock *flp, *flp2, *empty;
-
-  /* Fetch the flock structure from user space. */
-  fs_flock   = (vir_bytes) &flock;
-  flock_size = (vir_bytes) sizeof(struct flock);
-  user_flock = (vir_bytes) name1;
-  fs_addr = (char *) fs_flock;
-  r = rw_user(D, who, user_flock, flock_size, fs_addr, FROM_USER);
-  if (r != OK) return(EINVAL);
-
-  /* Make some error checks. */
-  ltype = flock.l_type;
-  mo = f->filp_mode;
-  if (ltype != F_UNLCK && ltype != F_RDLCK && ltype != F_WRLCK) return(EINVAL);
-  if (req == F_GETLK && ltype == F_UNLCK) return(EINVAL);
-  if ( (f->filp_ino->i_mode & I_TYPE) != I_REGULAR) return(EINVAL);
-  if (req != F_GETLK && ltype == F_RDLCK && (mo & R_BIT) == 0) return(EBADF);
-  if (req != F_GETLK && ltype == F_WRLCK && (mo & W_BIT) == 0) return(EBADF);
-
-  /* Compute the first and last bytes in the lock region. */
-  switch (flock.l_whence) {
-	case SEEK_SET:	first = 0; break;
-	case SEEK_CUR:	first = f->filp_pos; break;
-	case SEEK_END:	first = f->filp_ino->i_size; break;
-	default:	return(EINVAL);
-  }
-  /* check for overflow */
-  if (((long)flock.l_start > 0) && ((first + flock.l_start) < first))
-	return(EINVAL);
-  if (((long)flock.l_start < 0) && ((first + flock.l_start) > first))
-	return(EINVAL);
-  first = first + flock.l_start;
-  last = first + flock.l_len - 1;
-  if (flock.l_len == 0) last = MAX_FILE_POS;
-  if (last < first) return(EINVAL);
-
-  /* Check if this region conflicts with any existing lock. */
-  empty = (struct file_lock *) 0;
-  for (flp = &file_lock[0]; flp < & file_lock[NR_LOCKS]; flp++) {
-	if (flp->lock_type == 0) {
-		if (empty == (struct file_lock *) 0) empty = flp;
-		continue;	/* 0 means unused slot */
-	}
-	if (flp->lock_inode != f->filp_ino) continue;	/* different file */
-	if (last < flp->lock_first) continue;	/* new one is in front */
-	if (first > flp->lock_last) continue;	/* new one is afterwards */
-	if (ltype == F_RDLCK && flp->lock_type == F_RDLCK) continue;
-	if (req != F_GETLK && ltype != F_UNLCK && flp->lock_pid == fp->fp_pid) 
-		continue;
-  
-	/* There might be a conflict.  Process it. */
-	conflict = 1;
-	if (req == F_GETLK) break;
-
-	/* If we are trying to set a lock, it just failed. */
-	if (ltype == F_RDLCK || ltype == F_WRLCK) {
-		if (req == F_SETLK) {
-			/* For F_SETLK, just report back failure. */
-			return(EAGAIN);
-		} else {
-			/* For F_SETLKW, suspend the process. */
-			suspend(XLOCK);
-			return(0);
-		}
-	}
-
-	/* We are clearing a lock and we found something that overlaps. */
-	unlocking = 1;
-	if (first <= flp->lock_first && last >= flp->lock_last) {
-		flp->lock_type = 0;	/* mark slot as unused */
-		nr_locks--;		/* number of locks is now 1 less */
-		continue;
-	}
-
-	/* Part of a locked region has been unlocked. */
-	if (first <= flp->lock_first) {
-		flp->lock_first = last + 1;
-		continue;
-	}
-
-	if (last >= flp->lock_last) {
-		flp->lock_last = first - 1;
-		continue;
-	}
-	
-	/* Bad luck. A lock has been split in two by unlocking the middle. */
-	if (nr_locks == NR_LOCKS) return(ENOLCK);
-	for (i = 0; i < NR_LOCKS; i++)
-		if (file_lock[i].lock_type == 0) break;
-	flp2 = &file_lock[i];
-	flp2->lock_type = flp->lock_type;
-	flp2->lock_pid = flp->lock_pid;
-	flp2->lock_inode = flp->lock_inode;
-	flp2->lock_first = last + 1;
-	flp2->lock_last = flp->lock_last;
-	flp->lock_last = first - 1;
-	nr_locks++;
-  }
-  if (unlocking) lock_revive();
-
-  if (req == F_GETLK) {
-	if (conflict) {
-		/* GETLK and conflict. Report on the conflicting lock. */
-		flock.l_type = flp->lock_type;
-		flock.l_whence = SEEK_SET;
-		flock.l_start = flp->lock_first;
-		flock.l_len = flp->lock_last - flp->lock_first + 1;
-		flock.l_pid = flp->lock_pid;
-
-	} else {
-		/* It is GETLK and there is no conflict. */
-		flock.l_type = F_UNLCK;
-	}
-
-	/* Copy the flock structure back to the caller. */
-	fs_addr = (char *) fs_flock;
-	r = rw_user(D, who, user_flock, flock_size, fs_addr, TO_USER);
-	return(r);
-  }
-
-  if (ltype == F_UNLCK) return(OK);	/* unlocked a region with no locks */
-
-  /* There is no conflict.  If space exists, store new lock in the table. */
-  if (empty == (struct file_lock *) 0) return(ENOLCK);	/* table full */
-  empty->lock_type = ltype;
-  empty->lock_pid = fp->fp_pid;
-  empty->lock_inode = f->filp_ino;
-  empty->lock_first = first;
-  empty->lock_last = last;
-  nr_locks++;
-  return(OK);
-}
-
-/*===========================================================================*
- *				lock_revive				     *
- *===========================================================================*/
-PUBLIC void lock_revive()
-{
-/* Go find all the processes that are waiting for any kind of lock and 
- * revive them all.  The ones that are still blocked will block again when 
- * they run.  The others will complete.  This strategy is a space-time 
- * tradeoff.  Figuring out exactly which ones to unblock now would take 
- * extra code, and the only thing it would win would be some performance in 
- * extremely rare circumstances (namely, that somebody actually used 
- * locking).
- */
-
-  int task;
-  struct fproc *fptr;
-
-  for (fptr = &fproc[INIT_PROC_NR + 1]; fptr < &fproc[NR_PROCS]; fptr++){
-	task = -fptr->fp_task;
-	if (fptr->fp_suspended == SUSPENDED && task == XLOCK) {
-		revive((int)(fptr - fproc), 0);
-	}
-  }
-}
-
 /*===========================================================================*
  *				do_sync					     *
  *===========================================================================*/
@@ -340,28 +164,24 @@ PUBLIC int do_fork()
  */
 
   register struct fproc *cp;
-  register char *sptr, *dptr;
   int i;
 
   /* Only MM may make this call directly. */
-  if (who != MM_PROC_NR) return(ERROR);
+  if (who != MM_PROC_NR) return(EGENERIC);
 
   /* Copy the parent's fproc struct to the child. */
-  sptr = (char *) &fproc[parent];	/* pointer to parent's 'fproc' struct*/
-  dptr = (char *) &fproc[child];	/* pointer to child's 'fproc' struct */
-  i = sizeof(struct fproc);		/* how many bytes to copy */
-  while (i--) *dptr++ = *sptr++;	/* fproc[child] = fproc[parent] */
+  fproc[child] = fproc[parent];
 
   /* Increase the counters in the 'filp' table. */
   cp = &fproc[child];
   for (i = 0; i < OPEN_MAX; i++)
 	if (cp->fp_filp[i] != NIL_FILP) cp->fp_filp[i]->filp_count++;
 
-  /* Fill in new process id and, if necessary, process group. */
+  /* Fill in new process id. */
   cp->fp_pid = pid;
-  if (parent == INIT_PROC_NR) {
-	cp->fp_pgrp = pid;
-  }
+
+  /* A child is not a process leader. */
+  cp->fp_sesldr = 0;
 
   /* Record the fact that both root and working dir have another user. */
   dup_inode(cp->fp_rootdir);
@@ -383,7 +203,7 @@ PUBLIC int do_exec()
   long bitmap;
 
   /* Only MM may make this call directly. */
-  if (who != MM_PROC_NR) return(ERROR);
+  if (who != MM_PROC_NR) return(EGENERIC);
 
   /* The array of FD_CLOEXEC bits is in the fp_cloexec bit map. */
   fp = &fproc[slot1];		/* get_filp() needs 'fp' */
@@ -408,9 +228,15 @@ PUBLIC int do_exit()
 /* Perform the file system portion of the exit(status) system call. */
 
   register int i, exitee, task;
+  register struct fproc *rfp;
+  register struct filp *rfilp;
+  register struct inode *rip;
+  int major;
+  dev_t dev;
+  message dev_mess;
 
   /* Only MM may do the EXIT call directly. */
-  if (who != MM_PROC_NR) return(ERROR);
+  if (who != MM_PROC_NR) return(EGENERIC);
 
   /* Nevertheless, pretend that the call came from the user. */
   fp = &fproc[slot1];		/* get_filp() needs 'fp' */
@@ -436,6 +262,31 @@ PUBLIC int do_exit()
   fp->fp_rootdir = NIL_INODE;
   fp->fp_workdir = NIL_INODE;
 
+  /* If a session leader exits then revoke access to its controlling tty from
+   * all other processes using it.
+   */
+  if (!fp->fp_sesldr) return(OK);		/* not a session leader */
+  fp->fp_sesldr = FALSE;
+  if (fp->fp_tty == 0) return(OK);		/* no controlling tty */
+  dev = fp->fp_tty;
+
+  for (rfp = &fproc[LOW_USER]; rfp < &fproc[NR_PROCS]; rfp++) {
+	if (rfp->fp_tty == dev) rfp->fp_tty = 0;
+
+	for (i = 0; i < OPEN_MAX; i++) {
+		if ((rfilp = rfp->fp_filp[i]) == NIL_FILP) continue;
+		if (rfilp->filp_mode == FILP_CLOSED) continue;
+		rip = rfilp->filp_ino;
+		if ((rip->i_mode & I_TYPE) != I_CHAR_SPECIAL) continue;
+		if ((dev_t) rip->i_zone[0] != dev) continue;
+		dev_mess.m_type = DEV_CLOSE;
+		dev_mess.DEVICE = dev;
+		major = (dev >> MAJOR) & BYTE;	/* major device nr */
+		task = dmap[major].dmap_task;	/* device task nr */
+		(*dmap[major].dmap_close)(task, &dev_mess);
+		rfilp->filp_mode = FILP_CLOSED;
+	}
+  }
   return(OK);
 }
 
@@ -450,7 +301,7 @@ PUBLIC int do_set()
   register struct fproc *tfp;
 
   /* Only MM may make this call directly. */
-  if (who != MM_PROC_NR) return(ERROR);
+  if (who != MM_PROC_NR) return(EGENERIC);
 
   tfp = &fproc[slot1];
   if (fs_call == SETUID) {
@@ -481,7 +332,7 @@ PUBLIC int do_revive()
  */
 
 #if !ALLOW_USER_SEND
-  if (who > 0) return(EPERM);
+  if (who >= LOW_USER) return(EPERM);
 #endif
 
   revive(m.REP_PROC_NR, m.REP_STATUS);
