@@ -12,10 +12,9 @@
 #include "file.h"
 #include "fproc.h"
 #include "inode.h"
+#include "lock.h"
 #include "super.h"
 
-PUBLIC char *stackpt = &fstack[FS_STACK_BYTES];	/* initial stack pointer */
-
 PUBLIC _PROTOTYPE (int (*call_vector[NCALLS]), (void) ) = {
 	no_sys,		/*  0 = unused	*/
 	do_exit,	/*  1 = exit	*/
@@ -79,12 +78,12 @@ PUBLIC _PROTOTYPE (int (*call_vector[NCALLS]), (void) ) = {
 	do_exec,	/* 59 = exece	*/
 	do_umask,	/* 60 = umask	*/
 	do_chroot,	/* 61 = chroot	*/
-	no_sys,		/* 62 = unused	*/
-	no_sys,		/* 63 = unused	*/
+	do_setsid,	/* 62 = setsid	*/
+	no_sys,		/* 63 = getpgrp	*/
 
 	no_sys,		/* 64 = KSIG: signals originating in the kernel	*/
 	do_unpause,	/* 65 = UNPAUSE	*/
-	no_sys, 	/* 66 = BRK2 (used to tell MM size of FS,INIT)	*/
+	no_sys, 	/* 66 = unused  */
 	do_revive,	/* 67 = REVIVE	*/
 	no_sys,		/* 68 = TASK_REPLY	*/
 	no_sys,		/* 69 = unused */
@@ -94,9 +93,15 @@ PUBLIC _PROTOTYPE (int (*call_vector[NCALLS]), (void) ) = {
 	no_sys,		/* 73 = SIGPENDING */
 	no_sys,		/* 74 = SIGPROCMASK */
 	no_sys,		/* 75 = SIGRETURN */
+	no_sys,		/* 76 = REBOOT */
 };
 
 
+/* Some devices may or may not be there in the next table. */
+#define DT(enable, open, rw, close, task) \
+	{ (enable ? (open) : no_dev), (enable ? (rw) : no_dev), \
+	  (enable ? (close) : no_dev), (enable ? (task) : 0) },
+
 /* The order of the entries here determines the mapping between major device
  * numbers and tasks.  The first entry (major device 0) is not used.  The
  * next entry is major device 1, etc.  Character and block devices can be
@@ -107,21 +112,36 @@ PUBLIC _PROTOTYPE (int (*call_vector[NCALLS]), (void) ) = {
  * is changed from 1, NULL_MAJOR must be changed in <include/minix/com.h>.
  */
 PUBLIC struct dmap dmap[] = {
-/*  Open       Read/Write   Close       Task #      Device  File
-    ----       ----------   -----       -------     ------  ----      */
-    0,         0,           0,          0,           /* 0 = not used  */
-    dev_opcl,  call_task,   dev_opcl,   MEM,         /* 1 = /dev/mem  */
-    dev_opcl,  call_task,   dev_opcl,   FLOPPY,      /* 2 = /dev/fd0  */
-    dev_opcl,  call_task,   dev_opcl,   WINCHESTER,  /* 3 = /dev/hd0  */
-    tty_open,  call_task,   tty_close,  TTY,         /* 4 = /dev/tty0 */
-    ctty_open, rw_dev2,     ctty_close, TTY,         /* 5 = /dev/tty  */
-    no_call,   call_task,   no_call,    PRINTER,     /* 6 = /dev/lp   */
-    dev_opcl,  call_task,   dev_opcl,   SCSI,        /* 7 = /dev/hdscsi0  */
-#if NETWORKING_ENABLED
-    nw_open,   nw_rw,       nw_close,	INET_PROC_NR, /* 8 = /dev/ip   */
-#if ALLOW_USER_SEND
-    nw_open,   nw_rw,       nw_close,   7,            /* 9 = debug /dev/ip */
-#endif
+/*  ?   Open       Read/Write   Close       Task #      Device  File
+    -   ----       ----------   -----       -------     ------  ----       */
+  DT(1, no_dev,    no_dev,      no_dev,     0)           /* 0 = not used   */
+  DT(1, dev_opcl,  call_task,   dev_opcl,   MEM)         /* 1 = /dev/mem   */
+  DT(1, dev_opcl,  call_task,   dev_opcl,   FLOPPY)      /* 2 = /dev/fd0   */
+  DT(ENABLE_WINI,
+        dev_opcl,  call_task,   dev_opcl,   WINCHESTER)  /* 3 = /dev/hd0   */
+  DT(1, tty_open,  call_task,   dev_opcl,   TTY)         /* 4 = /dev/tty00 */
+  DT(1, ctty_open, call_ctty,   ctty_close, TTY)         /* 5 = /dev/tty   */
+  DT(1, dev_opcl,  call_task,   dev_opcl,    PRINTER)     /* 6 = /dev/lp    */
+
+#if (MACHINE == IBM_PC)
+  DT(ENABLE_NETWORKING,
+        net_open,  call_task,   dev_opcl,   INET_PROC_NR)/* 7 = /dev/ip    */
+  DT(ENABLE_CDROM,
+        dev_opcl,  call_task,   dev_opcl,   CDROM)       /* 8 = /dev/cd0   */
+  DT(0, 0,         0,           0,          0)           /* 9 = not used   */
+  DT(ENABLE_SCSI,
+        dev_opcl,  call_task,   dev_opcl,   SCSI)        /*10 = /dev/sd0   */
+  DT(0, 0,         0,           0,          0)           /*11 = not used   */
+  DT(0, 0,         0,           0,          0)           /*12 = not used   */
+  DT(ENABLE_AUDIO,
+        dev_opcl,  call_task,   dev_opcl,   AUDIO)       /*13 = /dev/audio */
+  DT(ENABLE_AUDIO,
+        dev_opcl,  call_task,   dev_opcl,   MIXER)       /*14 = /dev/mixer */
+#endif /* IBM_PC */
+
+#if (MACHINE == ATARI)
+  DT(ENABLE_SCSI,
+        dev_opcl,  call_task,   dev_opcl,   SCSI)        /* 7 = /dev/hdscsi0 */
 #endif
 };
 
