@@ -172,7 +172,7 @@ message *m_ptr;			/* pointer to message buffer */
   vir_clicks vlo, vhi;		/* virtual clicks containing message to send */
 
   /* User processes are only allowed to send to FS and MM.  Check for this. */
-  if (isuserp(caller_ptr) && !isservn(dest)) return(E_BAD_DEST);
+  if (isuserp(caller_ptr) && !issysentn(dest)) return(E_BAD_DEST);
   dest_ptr = proc_addr(dest);	/* pointer to destination's proc entry */
   if (dest_ptr->p_flags & P_SLOT_FREE) return(E_BAD_DEST);	/* dead dest */
 
@@ -186,7 +186,7 @@ message *m_ptr;			/* pointer to message buffer */
   vhi = (vb + MESS_SIZE - 1) >> CLICK_SHIFT;	/* vir click for top of msg */
   if (vlo < caller_ptr->p_map[D].mem_vir || vlo > vhi ||
       vhi >= caller_ptr->p_map[S].mem_vir + caller_ptr->p_map[S].mem_len)
-        return(E_BAD_ADDR); 
+        return(EFAULT); 
 #else
   /* Check for messages wrapping around top of memory or outside data seg. */
   vb = (vir_bytes) m_ptr;
@@ -194,7 +194,7 @@ message *m_ptr;			/* pointer to message buffer */
   vhi = (vb + MESS_SIZE - 1) >> CLICK_SHIFT;	/* vir click for top of msg */
   if (vhi < vlo ||
       vhi - caller_ptr->p_map[D].mem_vir >= caller_ptr->p_map[D].mem_len)
-	return(E_BAD_ADDR);
+	return(EFAULT);
 #endif
 
   /* Check for deadlock by 'caller_ptr' and 'dest' sending to each other. */
@@ -404,6 +404,10 @@ register struct proc *rp;	/* this process is no longer runnable */
   register struct proc **qtail;  /* TASK_Q, SERVER_Q, or USER_Q rdy_tail */
 
   if (istaskp(rp)) {
+	/* task stack still ok? */
+	if (*rp->p_stguard != STACK_GUARD)
+		panic("stack overrun by task", proc_number(rp));
+
 	if ( (xp = rdy_head[TASK_Q]) == NIL_PROC) return;
 	if (xp == rp) {
 		/* Remove head of queue */
@@ -456,8 +460,7 @@ register struct proc *rp;	/* this process is no longer runnable */
   while (xp->p_nextready != rp)
 	if ( (xp = xp->p_nextready) == NIL_PROC) return;
   xp->p_nextready = xp->p_nextready->p_nextready;
-  while (xp->p_nextready != NIL_PROC) xp = xp->p_nextready;
-  *qtail = xp;
+  if (*qtail == rp) *qtail = xp;
 }
 
 
