@@ -1,14 +1,19 @@
 /*==========================================================================*
- *		rs232.c - serial driver for 8250 and 16450 UARTs 	    *
+ *		rs232.c - serial driver for 8250 and 16450 UARTs	    *
  *		Added support for Atari ST M68901 and YM-2149	--kub	    *
  *==========================================================================*/
 
 #include "kernel.h"
+#include <termios.h>
 #include <sgtty.h>
+#include <signal.h>
 #include "tty.h"
+#include "proc.h"
 
-#if (CHIP != INTEL) && (MACHINE != ATARI)
-#error				/* rs232.c only supports PC/AT and Atari ST */
+#if NR_RS_LINES > 0
+
+#if (MACHINE != IBM_PC) && (MACHINE != ATARI)
+#error				/* rs232.c only supports PC and Atari ST */
 #endif
 
 #if (MACHINE == ATARI)
@@ -20,12 +25,7 @@
 #endif
 #endif
 
-/* Switches.
- * #define C_RS232_INT_HANDLERS to use the interrupt handlers in this file.
- * #define NO_HANDSHAKE to avoid requiring CTS for output.
- */
-
-#if (CHIP == INTEL)		/* PC/AT 8250/16450 chip combination */
+#if (MACHINE == IBM_PC)		/* PC/AT 8250/16450 chip combination */
 
 /* 8250 constants. */
 #define UART_FREQ         115200L	/* timer frequency */
@@ -43,12 +43,11 @@
 #define IS_LINE_STATUS_CHANGE   6
 
 /* Line control bits. */
-#define LC_NO_PARITY            0
-#define LC_DATA_BITS            3
-#define LC_ODD_PARITY           8
-#define LC_EVEN_PARITY       0x18
+#define LC_2STOP_BITS        0x04
+#define LC_PARITY            0x08
+#define LC_PAREVEN           0x10
+#define LC_BREAK             0x40
 #define LC_ADDRESS_DIVISOR   0x80
-#define LC_STOP_BITS_SHIFT      2
 
 /* Line status bits. */
 #define LS_OVERRUN_ERR          2
@@ -64,6 +63,8 @@
 
 /* Modem status bits. */
 #define MS_CTS               0x10
+#define MS_RLSD              0x80       /* Received Line Signal Detect */
+#define MS_DRLSD             0x08       /* RLSD Delta */
 
 #else /* MACHINE == ATARI */		/* Atari ST 68901 USART */
 
@@ -73,12 +74,6 @@
 
 #define UART_FREQ          19200L	/* timer frequency */
 
-/* Line control bits. */
-#define LC_NO_PARITY            0
-#define LC_DATA_BITS            3
-#define LC_ODD_PARITY       U_PAR
-#define LC_EVEN_PARITY     (U_PAR|U_EVEN)
-
 /* Line status bits. */
 #define LS_OVERRUN_ERR       R_OE
 #define LS_PARITY_ERR        R_PE
@@ -88,26 +83,29 @@
 /* Modem status bits. */
 #define MS_CTS               IO_SCTS	/* 0x04 */
 
-#endif /* CHIP/MACHINE */
+#endif /* MACHINE == ATARI */
 
 #define DATA_BITS_SHIFT         8	/* amount data bits shifted in mode */
 #define DEF_BAUD             1200	/* default baud rate */
 
+#define RS_IBUFSIZE          1024	/* RS232 input buffer size */
+#define RS_OBUFSIZE          1024	/* RS232 output buffer size */
+
 /* Input buffer watermarks.
  * The external device is asked to stop sending when the buffer
- * exactly reaches high water, or when TTY requests it.
- * TTY is also notified directly (rather than at a later clock tick) when
- * this watermark is reached.
- * A lower threshold could be used, but the buffer size and wakeup intervals
- * are chosen so the watermark shouldn't be hit at reasonable baud rates,
- * so this is unnecessary - the throttle is applied when TTY's buffers
- * get too full.
- * The low watermark is invisibly 0 since the buffer is always emptied all
- * at once.
+ * exactly reaches high water, or when TTY requests it.  Sending restarts
+ * when the input buffer empties below the low watermark.
  */
-#define RS_IHIGHWATER (3 * RS_IBUFSIZE / 4)
+#define RS_ILOWWATER	(1 * RS_IBUFSIZE / 4)
+#define RS_IHIGHWATER	(3 * RS_IBUFSIZE / 4)
 
-#if (CHIP == INTEL)
+/* Output buffer low watermark.
+ * TTY is notified when the output buffer empties below the low watermark, so
+ * it may continue filling the buffer if doing a large write.
+ */
+#define RS_OLOWWATER	(1 * RS_OBUFSIZE / 4)
+
+#if (MACHINE == IBM_PC)
 
 /* Macros to handle flow control.
  * Interrupts must be off when they are used.
@@ -120,26 +118,30 @@
  * OUT2 is also kept high all the time.
  */
 #define istart(rs) \
-  (out_byte( (rs)->modem_ctl_port, MC_OUT2 | MC_RTS | MC_DTR), \
-   (rs)->idevready = TRUE)
+	(out_byte((rs)->modem_ctl_port, MC_OUT2 | MC_RTS | MC_DTR), \
+		(rs)->idevready = TRUE)
 #define istop(rs) \
-  (out_byte( (rs)->modem_ctl_port, MC_OUT2 | MC_DTR), (rs)->idevready = FALSE)
+	(out_byte((rs)->modem_ctl_port, MC_OUT2 | MC_DTR), \
+		(rs)->idevready = FALSE)
 
-/* Macro to tell if device is ready.
- * Don't require DSR, since modems drop this to indicate the line is not
- * ready even when the modem itself is ready.
- * If NO_HANDSHAKE, read the status port to clear the interrupt, then force
- * the ready bit.
+/* Macro to tell if device is ready.  The rs->cts field is set to MS_CTS if
+ * CLOCAL is in effect for a line without a CTS wire.
  */
-#if NO_HANDSHAKE
-# define devready(rs) (in_byte(rs->modem_status_port), MS_CTS)
-#else
-# define devready(rs) (in_byte(rs->modem_status_port) & MS_CTS)
-#endif
+#define devready(rs) ((in_byte(rs->modem_status_port) | rs->cts) & MS_CTS)
 
 /* Macro to tell if transmitter is ready. */
 #define txready(rs) (in_byte(rs->line_status_port) & LS_TRANSMITTER_READY)
 
+/* Macro to tell if carrier has dropped.
+ * The RS232 Carrier Detect (CD) line is usually connected to the 8250
+ * Received Line Signal Detect pin, reflected by bit MS_RLSD in the Modem
+ * Status Register.  The MS_DRLSD bit tells if MS_RLSD has just changed state.
+ * So if MS_DRLSD is set and MS_RLSD cleared, we know that carrier has just
+ * dropped.
+ */
+#define devhup(rs)	\
+	(in_byte(rs->modem_status_port) & (MS_RLSD|MS_DRLSD) == MS_DRLSD)
+
 #else /* MACHINE == ATARI */
 
 /* Macros to handle flow control.
@@ -160,51 +162,51 @@
 #define istop(rs)          { set_porta( ~PA_SDTR, PA_SRTS );	\
 			     (rs)->idevready = FALSE;	}
 
-/* Macro to tell if device is ready.
- * Don't require DSR, since modems drop this to indicate the line is not
- * ready even when the modem itself is ready.
- * If NO_HANDSHAKE, force the ready bit.
+/* Macro to tell if device is ready.  The rs->cts field is set to MS_CTS if
+ * CLOCAL is in effect for a line without a CTS wire.
  */
-#if NO_HANDSHAKE
-#define devready(rs)         MS_CTS
-#else
-#define devready(rs)         (~MFP->mf_gpip & MS_CTS)
-#endif
+#define devready(rs)         ((~MFP->mf_gpip | rs->cts) & MS_CTS)
 
 /* Transmitter ready test */
 #define txready(rs)          (MFP->mf_tsr & (T_EMPTY | T_UE))
 
-#endif /* CHIP/MACHINE */
+#endif /* MACHINE == ATARI */
 
 /* Types. */
 typedef unsigned char bool_t;	/* boolean */
 
 /* RS232 device structure, one per device. */
-struct rs232_s {
-  int minor;			/* minor number of this line (base 0) */
+typedef struct rs232 {
+  tty_t *tty;			/* associated TTY structure */
 
+  int icount;			/* number of bytes in the input buffer */
+  char *ihead;			/* next free spot in input buffer */
+  char *itail;			/* first byte to give to TTY */
   bool_t idevready;		/* nonzero if we are ready to receive (RTS) */
-  bool_t ittyready;		/* nonzero if TTY is ready to receive */
-  char *ibuf;			/* start of input buffer */
-  char *ibufend;		/* end of input buffer */
-  char *ihighwater;		/* threshold in input buffer */
-  char *iptr;			/* next free spot in input buffer */
+  char cts;			/* normally 0, but MS_CTS if CLOCAL is set */
 
   unsigned char ostate;		/* combination of flags: */
-#define ODEVREADY MS_CTS	/* external device hardware ready (CTS) */
 #define ODONE          1	/* output completed (< output enable bits) */
-#define OQUEUED     0x20	/* output buffer not empty */
 #define ORAW           2	/* raw mode for xoff disable (< enab. bits) */
-#define OSWREADY    0x40	/* external device software ready (no xoff) */
 #define OWAKEUP        4	/* tty_wakeup() pending (asm code only) */
-#if (ODEVREADY | 0x63) == 0x63
+#define ODEVREADY MS_CTS	/* external device hardware ready (CTS) */
+#define OQUEUED     0x20	/* output buffer not empty */
+#define OSWREADY    0x40	/* external device software ready (no xoff) */
+#define ODEVHUP  MS_RLSD	/* external device has dropped carrier */
+#define OSOFTBITS  (ODONE | ORAW | OWAKEUP | OQUEUED | OSWREADY)
+				/* user-defined bits */
+#if (OSOFTBITS | ODEVREADY | ODEVHUP) == OSOFTBITS
+				/* a weak sanity check */
 #error				/* bits are not unique */
 #endif
   unsigned char oxoff;		/* char to stop output */
-  char *obufend;		/* end of output buffer */
-  char *optr;			/* next char to output */
+  bool_t inhibited;		/* output inhibited? (follows tty_inhibited) */
+  bool_t drain;			/* if set drain output and reconfigure line */
+  int ocount;			/* number of bytes in the output buffer */
+  char *ohead;			/* next free spot in output buffer */
+  char *otail;			/* next char to output */
 
-#if (CHIP == INTEL)
+#if (MACHINE == IBM_PC)
   port_t xmit_port;		/* i/o ports */
   port_t recv_port;
   port_t div_low_port;
@@ -224,18 +226,18 @@ struct rs232_s {
   unsigned parity_errors;
   unsigned break_interrupts;
 
-  char ibuf1[RS_IBUFSIZE + 1];	/* 1st input buffer, guard at end */
-  char ibuf2[RS_IBUFSIZE + 1];	/* 2nd input buffer (for swapping) */
-};
+  char ibuf[RS_IBUFSIZE];	/* input buffer */
+  char obuf[RS_OBUFSIZE];	/* output buffer */
+} rs232_t;
 
-/* Table and macro to translate an RS232 minor device number to its
- * struct rs232_s pointer.
- */
-struct rs232_s *p_rs_addr[NR_RS_LINES];
+PUBLIC rs232_t rs_lines[NR_RS_LINES];
 
-#define rs_addr(minor) (p_rs_addr[minor])
+/* Table and macro to translate an RS232 line number to its rs_lines entry. */
+PRIVATE rs232_t *p_rs_addr[NR_RS_LINES];
 
-#if (CHIP == INTEL)
+#define rs_addr(line)	(p_rs_addr[line])
+
+#if (MACHINE == IBM_PC)
 /* 8250 base addresses. */
 PRIVATE port_t addr_8250[] = {
   0x3F8,			/* COM1: (line 0); COM3 might be at 0x3E8 */
@@ -243,59 +245,194 @@ PRIVATE port_t addr_8250[] = {
 };
 #endif
 
-PUBLIC struct rs232_s rs_lines[NR_RS_LINES];
+FORWARD _PROTOTYPE( int rs232_1handler, (int irq)			);
+FORWARD _PROTOTYPE( int rs232_2handler, (int irq)			);
+FORWARD _PROTOTYPE( void in_int, (rs232_t *rs)				);
+FORWARD _PROTOTYPE( void line_int, (rs232_t *rs)			);
+FORWARD _PROTOTYPE( void modem_int, (rs232_t *rs)			);
+FORWARD _PROTOTYPE( void rs_write, (tty_t *tp)				);
+FORWARD _PROTOTYPE( void rs_echo, (tty_t *tp, int c)			);
+FORWARD _PROTOTYPE( void rs_ioctl, (tty_t *tp)				);
+FORWARD _PROTOTYPE( void rs_config, (rs232_t *rs)			);
+FORWARD _PROTOTYPE( void rs_read, (tty_t *tp)				);
+FORWARD _PROTOTYPE( void rs_icancel, (tty_t *tp)			);
+FORWARD _PROTOTYPE( void rs_ocancel, (tty_t *tp)			);
+FORWARD _PROTOTYPE( void rs_ostart, (rs232_t *rs)			);
+FORWARD _PROTOTYPE( void rs_break, (tty_t *tp)				);
+FORWARD _PROTOTYPE( void out_int, (rs232_t *rs)				);
 
-#if C_RS232_INT_HANDLERS
-FORWARD _PROTOTYPE( void in_int, (struct rs232_s *rs) );
-FORWARD _PROTOTYPE( void line_int, (struct rs232_s *rs) );
-FORWARD _PROTOTYPE( void modem_int, (struct rs232_s *rs) );
-#endif
-
-FORWARD _PROTOTYPE( void out_int, (struct rs232_s *rs) );
-FORWARD _PROTOTYPE( int rs_config, (int minor, int in_baud, int out_baud,
-		int parity, int stop_bits, int data_bits, int mode) );
-
-
-/* High level routines (should only be called by TTY). */
 
 /*==========================================================================*
- *				rs_config			 	    *
+ *				rs_write				    *
  *==========================================================================*/
-PRIVATE int rs_config(minor, in_baud, out_baud, parity, stop_bits, data_bits,
-		      mode)
-int minor;			/* which rs line */
-int in_baud;			/* input speed: 110, 300, 1200, etc */
-int out_baud;			/* output speed: 110, 300, 1200, etc */
-int parity;			/* LC_something */
-int stop_bits;			/* 2 (110 baud) or 1 (other speeds) */
-int data_bits;			/* 5, 6, 7, or 8 */
-int mode;			/* sgtty.h sg_mode word */
+PRIVATE void rs_write(tp)
+register tty_t *tp;
+{
+/* (*devwrite)() routine for RS232. */
+
+  rs232_t *rs = tp->tty_priv;
+  int count, ocount;
+  phys_bytes user_phys;
+
+  if (rs->inhibited != tp->tty_inhibited) {
+	/* Inhibition state has changed. */
+	lock();
+	rs->ostate |= OSWREADY;
+	if (tp->tty_inhibited) rs->ostate &= ~OSWREADY;
+	unlock();
+	rs->inhibited = tp->tty_inhibited;
+  }
+
+  if (rs->drain) {
+	/* Wait for the line to drain then reconfigure and continue output. */
+	if (rs->ocount > 0) return;
+	rs->drain = FALSE;
+	rs_config(rs);
+  }
+
+  /* While there is something to do. */
+  for (;;) {
+	ocount = buflen(rs->obuf) - rs->ocount;
+	count = bufend(rs->obuf) - rs->ohead;
+	if (count > ocount) count = ocount;
+	if (count > tp->tty_outleft) count = tp->tty_outleft;
+	if (count == 0 || tp->tty_inhibited) return;
+
+	/* Copy from user space to the RS232 output buffer. */
+	user_phys = proc_vir2phys(proc_addr(tp->tty_outproc), tp->tty_out_vir);
+	phys_copy(user_phys, vir2phys(rs->ohead), (phys_bytes) count);
+
+	/* Perform output processing on the output buffer. */
+	out_process(tp, rs->obuf, rs->ohead, bufend(rs->obuf), &count, &ocount);
+	if (count == 0) break;
+
+	/* Assume echoing messed up by output. */
+	tp->tty_reprint = TRUE;
+
+	/* Bookkeeping. */
+	lock();			/* protect interrupt sensitive rs->ocount */
+	rs->ocount += ocount;
+	rs_ostart(rs);
+	unlock();
+	if ((rs->ohead += ocount) >= bufend(rs->obuf))
+		rs->ohead -= buflen(rs->obuf);
+	tp->tty_out_vir += count;
+	tp->tty_outcum += count;
+	if ((tp->tty_outleft -= count) == 0) {
+		/* Output is finished, reply to the writer. */
+		tty_reply(tp->tty_outrepcode, tp->tty_outcaller,
+					tp->tty_outproc, tp->tty_outcum);
+		tp->tty_outcum = 0;
+	}
+  }
+}
+
+
+/*==========================================================================*
+ *				rs_echo					    *
+ *==========================================================================*/
+PRIVATE void rs_echo(tp, c)
+tty_t *tp;			/* which TTY */
+int c;				/* character to echo */
+{
+/* Echo one character.  (Like rs_write, but only one character, optionally.) */
+
+  rs232_t *rs = tp->tty_priv;
+  int count, ocount;
+
+  ocount = buflen(rs->obuf) - rs->ocount;
+  if (ocount == 0) return;		/* output buffer full */
+  count = 1;
+  *rs->ohead = c;			/* add one character */
+
+  out_process(tp, rs->obuf, rs->ohead, bufend(rs->obuf), &count, &ocount);
+  if (count == 0) return;
+
+  lock();
+  rs->ocount += ocount;
+  rs_ostart(rs);
+  unlock();
+  if ((rs->ohead += ocount) >= bufend(rs->obuf)) rs->ohead -= buflen(rs->obuf);
+}
+
+
+/*==========================================================================*
+ *				rs_ioctl				    *
+ *==========================================================================*/
+PRIVATE void rs_ioctl(tp)
+tty_t *tp;			/* which TTY */
+{
+/* Reconfigure the line as soon as the output has drained. */
+  rs232_t *rs = tp->tty_priv;
+
+  rs->drain = TRUE;
+}
+
+
+/*==========================================================================*
+ *				rs_config				    *
+ *==========================================================================*/
+PRIVATE void rs_config(rs)
+rs232_t *rs;			/* which line */
 {
 /* Set various line control parameters for RS232 I/O.
  * If DataBits == 5 and StopBits == 2, 8250 will generate 1.5 stop bits.
- * The 8250 can't handle split speed, but we have propagated both speeds
- * anyway for the benefit of future UART chips.
+ * The 8250 can't handle split speed, so we use the input speed.
  */
 
+  tty_t *tp = rs->tty;
   int divisor;
   int line_controls;
-  register struct rs232_s *rs;
+  static struct speed2divisor {
+	speed_t	speed;
+	int	divisor;
+  } s2d[] = {
+#if (MACHINE == IBM_PC)
+	{ B50,		UART_FREQ / 50		},
+#endif
+	{ B75,		UART_FREQ / 75		},
+	{ B110,		UART_FREQ / 110		},
+	{ B134,		UART_FREQ * 10 / 1345	},
+	{ B150,		UART_FREQ / 150		},
+	{ B200,		UART_FREQ / 200		},
+	{ B300,		UART_FREQ / 300		},
+	{ B600,		UART_FREQ / 600		},
+	{ B1200,	UART_FREQ / 1200	},
+#if (MACHINE == IBM_PC)
+	{ B1800,	UART_FREQ / 1800	},
+#endif
+	{ B2400,	UART_FREQ / 2400	},
+	{ B4800,	UART_FREQ / 4800	},
+	{ B9600,	UART_FREQ / 9600	},
+	{ B19200,	UART_FREQ / 19200	},
+#if (MACHINE == IBM_PC)
+	{ B38400,	UART_FREQ / 38400	},
+	{ B57600,	UART_FREQ / 57600	},
+	{ B115200,	UART_FREQ / 115200L	},
+#endif
+  };
+  struct speed2divisor *s2dp;
 
-  rs = rs_addr(minor);
+  /* RS232 needs to know the xoff character, and if CTS works. */
+  rs->oxoff = tp->tty_termios.c_cc[VSTOP];
+  rs->cts = (tp->tty_termios.c_cflag & CLOCAL) ? MS_CTS : 0;
 
-  /* Precalculate divisor and line_controls for reduced latency. */
-  if (in_baud < 50) in_baud = DEF_BAUD;	/* prevent divide overflow */
-  if (out_baud < 50) out_baud = DEF_BAUD;	/* prevent divide overflow */
-  divisor = (int) (UART_FREQ / in_baud);	/* 8250 can't hack 2 speeds */
-#if (CHIP == INTEL)
-  switch(in_baud) {		/* HACK */
-    case 19300: divisor = 1; break;	/* 115200 */
-    case 19400: divisor = 2; break;	/* 57600 */
-    case 19500: divisor = 3; break;	/* 38400 */
-    case 19600: divisor = 4; break;	/* 28800 */
+  /* Look up the 8250 rate divisor from the output speed. */
+  divisor = 0;
+  for (s2dp = s2d; s2dp < s2d + sizeof(s2d)/sizeof(s2d[0]); s2dp++) {
+	if (s2dp->speed == tp->tty_termios.c_ospeed) divisor = s2dp->divisor;
   }
-  line_controls = parity | ((stop_bits - 1) << LC_STOP_BITS_SHIFT)
-			 | (data_bits - 5);
+  if (divisor == 0) return;	/* B0? */
+
+#if (MACHINE == IBM_PC)
+  /* Compute line control flag bits. */
+  line_controls = 0;
+  if (tp->tty_termios.c_cflag & PARENB) {
+	line_controls |= LC_PARITY;
+	if (!(tp->tty_termios.c_cflag & PARODD)) line_controls |= LC_PAREVEN;
+  }
+  if (divisor >= (UART_FREQ / 110)) line_controls |= LC_2STOP_BITS;
+  line_controls |= (tp->tty_termios.c_cflag & CSIZE) >> 2;
 
   /* Lock out interrupts while setting the speed. The receiver register is
    * going to be hidden by the div_low register, but the input interrupt
@@ -311,121 +448,63 @@ int mode;			/* sgtty.h sg_mode word */
 
   /* Change the line controls and reselect the usual registers. */
   out_byte(rs->line_ctl_port, line_controls);
+
+  rs->ostate |= ORAW;
+  if ((tp->tty_termios.c_lflag & IXON) && rs->oxoff != _POSIX_VDISABLE)
+	rs->ostate &= ~ORAW;
+
+  unlock();
+
 #else /* MACHINE == ATARI */
-  line_controls = parity | U_Q16;
-  switch (stop_bits) {
-	case 1:	line_controls |= U_ST1; break;
-	case 2:	line_controls |= U_ST2; break;
+
+  line_controls = U_Q16;
+  if (tp->tty_termios.c_cflag & PARENB) {
+	line_controls |= U_PAR;
+	if (!(tp->tty_termios.c_cflag & PARODD)) line_controls |= U_EVEN;
   }
-  switch (data_bits) {
-	case 5:	line_controls |= U_D5; break;
-	case 6:	line_controls |= U_D6; break;
-	case 7:	line_controls |= U_D7; break;
-	case 8:	line_controls |= U_D8; break;
+  line_controls |= (divisor >= (UART_FREQ / 110)) ? U_ST2 : U_ST1;
+
+  switch (tp->tty_termios.c_cflag & CSIZE) {	/* XXX - are U_Dn like CSn? */
+	case CS5:	line_controls |= U_D5; break;
+	case CS5:	line_controls |= U_D6; break;
+	case CS5:	line_controls |= U_D7; break;
+	case CS5:	line_controls |= U_D8; break;
   }
   lock();
   MFP->mf_ucr = line_controls;
-  /* Check if baud rate is valid. Some decent baud rates may not work because
-   * the timer cannot be programmed exactly enough, e.g. 7200. That is caught
-   * by the expressions below which check that the resulting baud rate has a
-   * deviation of max. 5%.
-   */
-  if (((UART_FREQ - (long)  divisor*in_baud >  UART_FREQ / 20) &&
-       (UART_FREQ - (long)++divisor*in_baud < -UART_FREQ / 20))   ||
-      divisor == 0 || divisor > 256)
-	printf ("tty line %d: can't set speed %d\n", minor, in_baud);
-  else
-	MFP->mf_tddr = divisor;
-#endif /* CHIP/MACHINE */
-
-  if (mode & RAW)
-	rs->ostate |= ORAW;
-  else
-	rs->ostate &= ~ORAW;
-  unlock();
-  return( (out_baud / 100) << 8) | (in_baud / 100);
-}
-
-
-/*==========================================================================*
- *				rs_ioctl			 	    *
- *==========================================================================*/
-PUBLIC int rs_ioctl(minor, mode, speeds)
-int minor;			/* which rs line */
-int mode;			/* sgtty.h sg_mode word */
-int speeds;			/* low byte is input speed, next is output */
-{
-/* Set the UART parameters. */
-
-  int data_bits;
-  int in_baud;
-  int out_baud;
-  int parity;
-  int stop_bits;
-
-  in_baud = 100 * (speeds & BYTE);
-  if (in_baud == 100) in_baud = 110;
-  out_baud = 100 * ((speeds >> 8) & BYTE);
-  if (out_baud == 100) out_baud = 110;
-  parity = LC_NO_PARITY;
-  if (mode & ODDP) parity = LC_ODD_PARITY;
-  if (mode & EVENP) parity = LC_EVEN_PARITY;
-  stop_bits = in_baud == 110 ? 2 : 1;	/* not quite cricket */
-  data_bits = 5 + ((mode >> DATA_BITS_SHIFT) & LC_DATA_BITS);
-  return(rs_config(minor, in_baud, out_baud, parity, stop_bits, data_bits,
-		     mode));
-}
-
-
-/*==========================================================================*
- *				rs_inhibit				    *
- *==========================================================================*/
-PUBLIC void rs_inhibit(minor, inhibit)
-int minor;			/* which rs line */
-bool_t inhibit;			/* nonzero to inhibit, zero to uninhibit */
-{
-/* Update inhibition state to keep in sync with TTY. */
-
-  register struct rs232_s *rs;
-
-  rs = rs_addr(minor);
-  lock();
-  if (inhibit)
-	rs->ostate &= ~OSWREADY;
-  else
-	rs->ostate |= OSWREADY;
+  MFP->mf_tddr = divisor;
   unlock();
+#endif /* MACHINE == ATARI */
 }
 
 
 /*==========================================================================*
  *				rs_init					    *
  *==========================================================================*/
-PUBLIC int rs_init(minor)
-int minor;			/* which rs line */
+PUBLIC void rs_init(tp)
+tty_t *tp;			/* which TTY */
 {
 /* Initialize RS232 for one line. */
 
-  register struct rs232_s *rs;
-  int speed;
-#if (CHIP == INTEL)
+  register rs232_t *rs;
+  int line;
+#if (MACHINE == IBM_PC)
   port_t this_8250;
+  int irq;
+  long v;
 #endif
 
-  rs = rs_addr(minor) = &rs_lines[minor];
-
-  /* Record minor number. */
-  rs->minor = minor;
+  /* Associate RS232 and TTY structures. */
+  line = tp - &tty_table[NR_CONS];
+  rs = tp->tty_priv = &rs_lines[line];
+  rs->tty = tp;
 
   /* Set up input queue. */
-  rs->iptr = rs->ibuf = rs->ibuf1;
-  rs->ibufend = rs->ibuf1 + RS_IBUFSIZE;
-  rs->ihighwater = rs->ibuf1 + RS_IHIGHWATER;
-  rs->ittyready = TRUE;		/* idevready set to TRUE by istart() */
+  rs->ihead = rs->itail = rs->ibuf;
 
-#if (CHIP == INTEL)
+#if (MACHINE == IBM_PC)
   /* Precalculate port numbers for speed. Magic numbers in the code (once). */
-  this_8250 = addr_8250[minor];
+  this_8250 = addr_8250[line];
   rs->xmit_port = this_8250 + 0;
   rs->recv_port = this_8250 + 0;
   rs->div_low_port = this_8250 + 0;
@@ -444,7 +523,7 @@ int minor;			/* which rs line */
    *	  here and that caused trouble (no interrupts) in version 1.5 by
    *	  hiding the interrupt enable port in the next step, and worse trouble
    *	  (continual interrupts) in an old version by hiding the receiver
-   *	  port in the first interrupt.  Call rs_config() early to avoid this.
+   *	  port in the first interrupt.  Call rs_ioctl() early to avoid this.
    *	o disable interrupts at the chip level, to force an edge transition
    *	  on the 8259 line when interrupts are next enabled and active.
    *	  RS232 interrupts are guaranteed to be disabled now by the 8259
@@ -452,8 +531,8 @@ int minor;			/* which rs line */
    *	  handling a previous interrupt.
    */
   istop(rs);			/* sets modem_ctl_port */
-  speed = rs_config(minor, DEF_BAUD, DEF_BAUD, LC_NO_PARITY, 1, 8, RAW);
-#if (CHIP == INTEL)
+  rs_config(rs);
+#if (MACHINE == IBM_PC)
   out_byte(rs->int_enab_port, 0);
 #endif
 
@@ -461,18 +540,29 @@ int minor;			/* which rs line */
    * and will occur when interrupts are enabled anyway.  Set up the output
    * queue using the status from clearing the modem status interrupt.
    */
-#if (CHIP == INTEL)
+#if (MACHINE == IBM_PC)
   in_byte(rs->line_status_port);
   in_byte(rs->recv_port);
 #endif
   rs->ostate = devready(rs) | ORAW | OSWREADY;	/* reads modem_ctl_port */
+  rs->ohead = rs->otail = rs->obuf;
 
-#if (CHIP == INTEL)
+#if (MACHINE == IBM_PC)
   /* Enable interrupts for both interrupt controller and device. */
-  if (minor & 1)		/* COM2 on IRQ3 */
-	enable_irq(SECONDARY_IRQ);
-  else				/* COM1 on IRQ4 */
-	enable_irq(RS232_IRQ);
+  irq = (line & 1) ? SECONDARY_IRQ : RS232_IRQ;
+
+#if ENABLE_NETWORKING
+  /* The ethernet driver may steal the IRQ of an RS232 line. */
+  v = ETHER_IRQ;
+  switch (env_parse("DPETH0", "x:d:x", 1, &v, 0L, (long) NR_IRQ_VECTORS-1)) {
+  case EP_ON:
+  case EP_SET:
+	if (v == irq) return;		/* IRQ in use, don't configure line */
+  }
+#endif
+
+  put_irq_handler(irq, (line & 1) ? rs232_2handler : rs232_1handler);
+  enable_irq(irq);
   out_byte(rs->int_enab_port, IE_LINE_STATUS_CHANGE | IE_MODEM_STATUS_CHANGE
 				| IE_RECEIVER_READY | IE_TRANSMITTER_READY);
 #else /* MACHINE == ATARI */
@@ -488,51 +578,35 @@ int minor;			/* which rs line */
   MFP->mf_imra |= (IA_RRDY|IA_RERR|IA_TRDY|IA_TERR);
   MFP->mf_ierb |= (IB_SCTS|IB_SDCD);
   MFP->mf_imrb |= (IB_SCTS|IB_SDCD);
-#endif /* CHIP/MACHINE */
+#endif /* MACHINE == ATARI */
+
+  /* Fill in TTY function hooks. */
+  tp->tty_devread = rs_read;
+  tp->tty_devwrite = rs_write;
+  tp->tty_echo = rs_echo;
+  tp->tty_icancel = rs_icancel;
+  tp->tty_ocancel = rs_ocancel;
+  tp->tty_ioctl = rs_ioctl;
+  tp->tty_break = rs_break;
 
   /* Tell external device we are ready. */
   istart(rs);
-
-  return(speed);
 }
 
 
 /*==========================================================================*
- *				rs_istop				    *
+ *				rs_icancel				    *
  *==========================================================================*/
-PUBLIC void rs_istop(minor)
-int minor;			/* which rs line */
+PRIVATE void rs_icancel(tp)
+tty_t *tp;			/* which TTY */
 {
-/* TTY wants RS232 to stop input.
- * RS232 drops RTS but keeps accepting input until its buffer overflows.
- */
+/* Cancel waiting input. */
+  rs232_t *rs = tp->tty_priv;
 
-  register struct rs232_s *rs;
-
-  rs = rs_addr(minor);
   lock();
-  rs->ittyready = FALSE;
-  istop(rs);
-  unlock();
-}
-
-
-/*==========================================================================*
- *				rs_istart				    *
- *==========================================================================*/
-PUBLIC void rs_istart(minor)
-int minor;			/* which rs line */
-{
-/* TTY is ready for another buffer full of input from RS232.
- * RS232 raises RTS unless its own buffer is already too full.
- */
-
-  register struct rs232_s *rs;
-
-  rs = rs_addr(minor);
-  lock();
-  rs->ittyready = TRUE;
-  if (rs->iptr < rs->ihighwater) istart(rs);
+  rs->icount = 0;
+  rs->itail = rs->ihead;
+  istart(rs);
   unlock();
 }
 
@@ -540,17 +614,16 @@ int minor;			/* which rs line */
 /*==========================================================================*
  *				rs_ocancel				    *
  *==========================================================================*/
-PUBLIC void rs_ocancel(minor)
-int minor;			/* which rs line */
+PRIVATE void rs_ocancel(tp)
+tty_t *tp;			/* which TTY */
 {
 /* Cancel pending output. */
-
-  register struct rs232_s *rs;
+  rs232_t *rs = tp->tty_priv;
 
   lock();
-  rs = rs_addr(minor);
-  if (rs->ostate & ODONE) tty_events -= EVENT_THRESHOLD;
   rs->ostate &= ~(ODONE | OQUEUED);
+  rs->ocount = 0;
+  rs->otail = rs->ohead;
   unlock();
 }
 
@@ -558,91 +631,84 @@ int minor;			/* which rs line */
 /*==========================================================================*
  *				rs_read					    *
  *==========================================================================*/
-PUBLIC int rs_read(minor, bufindirect, odoneindirect)
-int minor;			/* which rs line */
-char **bufindirect;		/* where to return pointer to our buffer */
-bool_t *odoneindirect;		/* where to return output-done status */
+PRIVATE void rs_read(tp)
+tty_t *tp;			/* which tty */
 {
-/* Swap the input buffers, giving the old one to TTY, and restart input. */
+/* Process characters from the circular input buffer. */
 
-  register char *ibuf;
-  int nread;
-  register struct rs232_s *rs;
+  rs232_t *rs = tp->tty_priv;
+  int icount, count, ostate;
 
-  rs = rs_addr(minor);
-  *odoneindirect = rs->ostate & ODONE;
-  if (rs->iptr == (ibuf = rs->ibuf)) return(0);
-  *bufindirect = ibuf;
-  lock();
-  nread = rs->iptr - ibuf;
-  tty_events -= nread;
-  if (ibuf == rs->ibuf1)
-	ibuf = rs->ibuf2;
-  else
-	ibuf = rs->ibuf1;
-  rs->ibufend = ibuf + RS_IBUFSIZE;
-  rs->ihighwater = ibuf + RS_IHIGHWATER;
-  rs->iptr = ibuf;
-  if (!rs->idevready && rs->ittyready) istart(rs);
-  unlock();
-  rs->ibuf = ibuf;
-  return(nread);
+  if (!(tp->tty_termios.c_cflag & CLOCAL)) {
+	/* Send a SIGHUP if hangup detected. */
+	lock();
+	ostate = rs->ostate;
+	rs->ostate &= ~ODEVHUP;		/* save ostate, clear DEVHUP */
+	unlock();
+	if (ostate & ODEVHUP) { sigchar(tp, SIGHUP); return; }
+  }
+
+  while ((count = rs->icount) > 0) {
+	icount = bufend(rs->ibuf) - rs->itail;
+	if (count > icount) count = icount;
+
+	/* Perform input processing on (part of) the input buffer. */
+	if ((count = in_process(tp, rs->itail, count)) == 0) break;
+
+	lock();			/* protect interrupt sensitive variables */
+	rs->icount -= count;
+	if (!rs->idevready && rs->icount < RS_ILOWWATER) istart(rs);
+	unlock();
+	if ((rs->itail += count) == bufend(rs->ibuf)) rs->itail = rs->ibuf;
+  }
 }
 
 
 /*==========================================================================*
- *				rs_setc					    *
+ *				rs_ostart				    *
  *==========================================================================*/
-PUBLIC void rs_setc(minor, xoff)
-int minor;			/* which rs line */
-int xoff;			/* xoff character */
+PRIVATE void rs_ostart(rs)
+rs232_t *rs;			/* which rs line */
 {
-/* RS232 needs to know the xoff character. */
+/* Tell RS232 there is something waiting in the output buffer. */
 
-  rs_addr(minor)->oxoff = xoff;
-}
-
-
-/*==========================================================================*
- *				rs_write				    *
- *==========================================================================*/
-PUBLIC void rs_write(minor, buf, nbytes)
-int minor;			/* which rs line */
-char *buf;			/* pointer to buffer to write */
-int nbytes;			/* number of bytes to write */
-{
-/* Tell RS232 about the buffer to be written and start output.
- * Previous output must have completed.
- */
-
-  register struct rs232_s *rs;
-
-  rs = rs_addr(minor);
-  lock();
-  rs->obufend = (rs->optr = buf) + nbytes;
   rs->ostate |= OQUEUED;
   if (txready(rs)) out_int(rs);
-  unlock();
+}
+
+
+/*==========================================================================*
+ *				rs_break				    *
+ *==========================================================================*/
+PRIVATE void rs_break(tp)
+tty_t *tp;			/* which tty */
+{
+/* Generate a break condition by setting the BREAK bit for 0.4 sec. */
+  rs232_t *rs = tp->tty_priv;
+  int line_controls;
+
+  line_controls = in_byte(rs->line_ctl_port);
+  out_byte(rs->line_ctl_port, line_controls | LC_BREAK);
+  milli_delay(400);						/* ouch */
+  out_byte(rs->line_ctl_port, line_controls);
 }
 
 
 /* Low level (interrupt) routines. */
 
-#if C_RS232_INT_HANDLERS
-#if (CHIP == INTEL)
+#if (MACHINE == IBM_PC)
 /*==========================================================================*
  *				rs232_1handler				    *
  *==========================================================================*/
-PUBLIC void rs232_1handler()
+PRIVATE int rs232_1handler(irq)
+int irq;
 {
 /* Interrupt hander for IRQ4.
  * Only 1 line (usually COM1) should use it.
  */
 
-#if NR_RS_LINES > 0
-  register struct rs232_s *rs;
+  register rs232_t *rs = &rs_lines[0];
 
-  rs = &rs_lines[0];
   while (TRUE) {
 	/* Loop to pick up ALL pending interrupts for device.
 	 * This usually just wastes time unless the hardware has a buffer
@@ -663,25 +729,23 @@ PUBLIC void rs232_1handler()
 		line_int(rs);
 		continue;
 	}
-	return;
+	return(1);	/* reenable serial interrupt */
   }
-#endif
 }
 
 
 /*==========================================================================*
  *				rs232_2handler				    *
  *==========================================================================*/
-PUBLIC void rs232_2handler()
+PRIVATE int rs232_2handler(irq)
+int irq;
 {
 /* Interrupt hander for IRQ3.
  * Only 1 line (usually COM2) should use it.
  */
 
-#if NR_RS_LINES > 1
-  register struct rs232_s *rs;
+  register rs232_t *rs = &rs_lines[1];
 
-  rs = &rs_lines[1];
   while (TRUE) {
 	switch (in_byte(rs->int_id_port)) {
 	case IS_RECEIVER_READY:
@@ -697,15 +761,14 @@ PUBLIC void rs232_2handler()
 		line_int(rs);
 		continue;
 	}
-	return;
+	return(1);	/* reenable serial interrupt */
   }
-#endif
 }
 #else /* MACHINE == ATARI */
 /*==========================================================================*
  *				siaint					    *
  *==========================================================================*/
-PUBLIC void siaint(type)
+PRIVATE void siaint(type)
 int    type;	       /* interrupt type */
 {
 /* siaint is the rs232 interrupt procedure for Atari ST's. For ST there are
@@ -714,7 +777,7 @@ int    type;	       /* interrupt type */
  */
 
   register unsigned char  code;
-  register struct rs232_s *rs = &rs_lines[0];
+  register rs232_t *rs = &rs_lines[0];
   int s = lock();
 
   switch (type & 0x00FF)
@@ -742,13 +805,14 @@ int    type;	       /* interrupt type */
   }
   restore(s);
 }
-#endif
+#endif /* MACHINE == ATARI */
+
 
 /*==========================================================================*
  *				in_int					    *
  *==========================================================================*/
 PRIVATE void in_int(rs)
-register struct rs232_s *rs;	/* line with input interrupt */
+register rs232_t *rs;		/* line with input interrupt */
 {
 /* Read the data which just arrived.
  * If it is the oxoff char, clear OSWREADY, else if OSWREADY was clear, set
@@ -757,24 +821,32 @@ register struct rs232_s *rs;	/* line with input interrupt */
  * Set a flag for the clock interrupt handler to eventually notify TTY.
  */
 
-#if (CHIP == INTEL)
-  if (rs->ostate & ORAW)
-	*rs->iptr = in_byte(rs->recv_port);
-  else if ( (*rs->iptr = in_byte(rs->recv_port)) == rs->oxoff)
-	rs->ostate &= ~OSWREADY;
+  int c;
+
+#if (MACHINE == IBM_PC)
+  c = in_byte(rs->recv_port);
 #else /* MACHINE == ATARI */
-  if (rs->ostate & ORAW)
-	*rs->iptr = MFP->mf_udr;
-  else if ( (*rs->iptr = MFP->mf_udr) == rs->oxoff)
-	rs->ostate &= ~OSWREADY;
-#endif /* CHIP/MACHINE */
-  else if (!(rs->ostate & OSWREADY)) {
-	rs->ostate |= OSWREADY;
-	if (txready(rs)) out_int(rs);
+  c = MFP->mf_udr;
+#endif
+
+  if (!(rs->ostate & ORAW)) {
+	if (c == rs->oxoff) {
+		rs->ostate &= ~OSWREADY;
+	} else
+	if (!(rs->ostate & OSWREADY)) {
+		rs->ostate |= OSWREADY;
+		if (txready(rs)) out_int(rs);
+	}
   }
-  if (rs->iptr < rs->ibufend) {
-	++tty_events;
-	if (++rs->iptr == rs->ihighwater) istop(rs);
+
+  if (rs->icount == buflen(rs->ibuf)) return;	/* input buffer full, discard */
+
+  if (++rs->icount == RS_IHIGHWATER && rs->idevready) istop(rs);
+  *rs->ihead = c;
+  if (++rs->ihead == bufend(rs->ibuf)) rs->ihead = rs->ibuf;
+  if (rs->icount == 1) {
+	rs->tty->tty_events = 1;
+	force_timeout();
   }
 }
 
@@ -783,17 +855,17 @@ register struct rs232_s *rs;	/* line with input interrupt */
  *				line_int				    *
  *==========================================================================*/
 PRIVATE void line_int(rs)
-register struct rs232_s *rs;	/* line with line status interrupt */
+register rs232_t *rs;		/* line with line status interrupt */
 {
 /* Check for and record errors. */
 
-#if (CHIP == INTEL)
+#if (MACHINE == IBM_PC)
   rs->lstatus = in_byte(rs->line_status_port);
 #else /* MACHINE == ATARI */
   rs->lstatus = MFP->mf_rsr;
   MFP->mf_rsr &= R_ENA;
   rs->pad = MFP->mf_udr;	/* discard char in case of LS_OVERRUN_ERR */
-#endif /* CHIP/MACHINE */
+#endif /* MACHINE == ATARI */
   if (rs->lstatus & LS_FRAMING_ERR) ++rs->framing_errors;
   if (rs->lstatus & LS_OVERRUN_ERR) ++rs->overrun_errors;
   if (rs->lstatus & LS_PARITY_ERR) ++rs->parity_errors;
@@ -805,7 +877,7 @@ register struct rs232_s *rs;	/* line with line status interrupt */
  *				modem_int				    *
  *==========================================================================*/
 PRIVATE void modem_int(rs)
-register struct rs232_s *rs;	/* line with modem interrupt */
+register rs232_t *rs;		/* line with modem interrupt */
 {
 /* Get possibly new device-ready status, and clear ODEVREADY if necessary.
  * If the device just became ready, restart output.
@@ -816,6 +888,13 @@ register struct rs232_s *rs;	/* line with modem interrupt */
   MFP->mf_aer = (MFP->mf_aer | (IO_SCTS|IO_SDCD)) ^
 		 (MFP->mf_gpip & (IO_SCTS|IO_SDCD));
 #endif
+
+  if (devhup(rs)) {
+	rs->ostate |= ODEVHUP;
+	rs->tty->tty_events = 1;
+	force_timeout();
+  }
+
   if (!devready(rs))
 	rs->ostate &= ~ODEVREADY;
   else if (!(rs->ostate & ODEVREADY)) {
@@ -824,37 +903,35 @@ register struct rs232_s *rs;	/* line with modem interrupt */
   }
 }
 
-#endif /* C_RS232_INT_HANDLERS (except out_int is used from high level) */
-
 
 /*==========================================================================*
  *				out_int					    *
  *==========================================================================*/
 PRIVATE void out_int(rs)
-register struct rs232_s *rs;	/* line with output interrupt */
+register rs232_t *rs;		/* line with output interrupt */
 {
 /* If there is output to do and everything is ready, do it (local device is
  * known ready).
- * Interrupt TTY to indicate completion.
+ * Notify TTY when the buffer goes empty.
  */
 
   if (rs->ostate >= (ODEVREADY | OQUEUED | OSWREADY)) {
 	/* Bit test allows ORAW and requires the others. */
-#if (CHIP == INTEL)
-	out_byte(rs->xmit_port, *rs->optr);
+#if (MACHINE == IBM_PC)
+	out_byte(rs->xmit_port, *rs->otail);
 #else /* MACHINE == ATARI */
-	MFP->mf_udr = *rs->optr;
+	MFP->mf_udr = *rs->otail;
 #endif
-	if (++rs->optr >= rs->obufend) {
-		tty_events += EVENT_THRESHOLD;
+	if (++rs->otail == bufend(rs->obuf)) rs->otail = rs->obuf;
+	if (--rs->ocount == 0) {
 		rs->ostate ^= (ODONE | OQUEUED);  /* ODONE on, OQUEUED off */
-#if (CHIP == INTEL)
-		unlock();	/* safe, for complicated reasons */
-		tty_wakeup();
-		lock();
-#else
-		tty_wakeup();	/* not save, for unknown reasons !?! */
-#endif
+		rs->tty->tty_events = 1;
+		force_timeout();
+	} else
+	if (rs->ocount == RS_OLOWWATER) {	/* running low? */
+		rs->tty->tty_events = 1;
+		force_timeout();
 	}
   }
 }
+#endif /* NR_RS_LINES > 0 */
