@@ -10,7 +10,9 @@
  *    - tell kernel about EXEC
  *    - save offset to initial argc (for ps)
  *
- *   The only entry point is do_exec.
+ * The entry points into this file are:
+ *   do_exec:	 perform the EXEC system call
+ *   find_share: find a process whose text segment can be shared
  */
 
 #include "mm.h"
@@ -18,13 +20,14 @@
 #include <minix/callnr.h>
 #include <a.out.h>
 #include <signal.h>
+#include <string.h>
 #include "mproc.h"
 #include "param.h"
 
 FORWARD _PROTOTYPE( void load_seg, (int fd, int seg, vir_bytes seg_bytes) );
-FORWARD _PROTOTYPE( int new_mem, (vir_bytes text_bytes, vir_bytes data_bytes,
-		vir_bytes bss_bytes, vir_bytes stk_bytes, phys_bytes tot_bytes,
-		char bf [ZEROBUF_SIZE ], int zs)			);
+FORWARD _PROTOTYPE( int new_mem, (struct mproc *sh_mp, vir_bytes text_bytes,
+		vir_bytes data_bytes, vir_bytes bss_bytes,
+		vir_bytes stk_bytes, phys_bytes tot_bytes)		);
 FORWARD _PROTOTYPE( void patch_ptr, (char stack [ARG_MAX ], vir_bytes base) );
 FORWARD _PROTOTYPE( int read_header, (int fd, int *ft, vir_bytes *text_bytes,
 		vir_bytes *data_bytes, vir_bytes *bss_bytes,
@@ -46,11 +49,11 @@ PUBLIC int do_exec()
  */
 
   register struct mproc *rmp;
+  struct mproc *sh_mp;
   int m, r, fd, ft, sn;
-  char mbuf[ARG_MAX];		/* buffer for stack and zeroes */
-  char name_buf[PATH_MAX];	/* the name of the file to exec */
-  char zb[ZEROBUF_SIZE];	/* used to zero bss */
-  char *new_sp;
+  static char mbuf[ARG_MAX];	/* buffer for stack and zeroes */
+  static char name_buf[PATH_MAX]; /* the name of the file to exec */
+  char *new_sp, *basename;
   vir_bytes src, dst, text_bytes, data_bytes, bss_bytes, stk_bytes, vsp;
   phys_bytes tot_bytes;		/* total space for program, including gap */
   long sym_bytes;
@@ -67,7 +70,8 @@ PUBLIC int do_exec()
   /* Get the exec file name and see if the file is executable. */
   src = (vir_bytes) exec_name;
   dst = (vir_bytes) name_buf;
-  r = mem_copy(who, D, (long) src, MM_PROC_NR, D, (long) dst, (long) exec_len);
+  r = sys_copy(who, D, (phys_bytes) src,
+		MM_PROC_NR, D, (phys_bytes) dst, (phys_bytes) exec_len);
   if (r != OK) return(r);	/* file name not in user data segment */
   tell_fs(CHDIR, who, FALSE, 0);	/* switch to the user's FS environ. */
   fd = allowed(name_buf, &s_buf, X_BIT);	/* is file executable? */
@@ -85,36 +89,50 @@ PUBLIC int do_exec()
   /* Fetch the stack from the user before destroying the old core image. */
   src = (vir_bytes) stack_ptr;
   dst = (vir_bytes) mbuf;
-  r = mem_copy(who, D, (long) src, MM_PROC_NR, D, (long) dst, (long)stk_bytes);
+  r = sys_copy(who, D, (phys_bytes) src,
+  			MM_PROC_NR, D, (phys_bytes) dst, (phys_bytes)stk_bytes);
   if (r != OK) {
 	close(fd);		/* can't fetch stack (e.g. bad virtual addr) */
 	return(EACCES);
   }
 
+  /* Can the process' text be shared with that of one already running? */
+  sh_mp = find_share(rmp, s_buf.st_ino, s_buf.st_dev, s_buf.st_ctime);
+
   /* Allocate new memory and release old memory.  Fix map and tell kernel. */
-  r = new_mem(text_bytes, data_bytes, bss_bytes, stk_bytes, tot_bytes,
-							zb, ZEROBUF_SIZE);
+  r = new_mem(sh_mp, text_bytes, data_bytes, bss_bytes, stk_bytes, tot_bytes);
   if (r != OK) {
 	close(fd);		/* insufficient core or program too big */
 	return(r);
   }
 
+  /* Save file identification to allow it to be shared. */
+  rmp->mp_ino = s_buf.st_ino;
+  rmp->mp_dev = s_buf.st_dev;
+  rmp->mp_ctime = s_buf.st_ctime;
+
   /* Patch up stack and copy it from MM to new core image. */
   vsp = (vir_bytes) rmp->mp_seg[S].mem_vir << CLICK_SHIFT;
   vsp += (vir_bytes) rmp->mp_seg[S].mem_len << CLICK_SHIFT;
   vsp -= stk_bytes;
   patch_ptr(mbuf, vsp);
   src = (vir_bytes) mbuf;
-  r = mem_copy(MM_PROC_NR, D, (long) src, who, D, (long) vsp, (long)stk_bytes);
+  r = sys_copy(MM_PROC_NR, D, (phys_bytes) src,
+  			who, D, (phys_bytes) vsp, (phys_bytes)stk_bytes);
   if (r != OK) panic("do_exec stack copy err", NO_NUM);
 
   /* Read in text and data segments. */
-  load_seg(fd, T, text_bytes);
+  if (sh_mp != NULL) {
+	lseek(fd, (off_t) text_bytes, SEEK_CUR);  /* shared: skip text */
+  } else {
+	load_seg(fd, T, text_bytes);
+  }
   load_seg(fd, D, data_bytes);
 
 #if (SHADOWING == 1)
-  if (lseek(fd, sym_bytes, 1) < 0) 	;		/* error */
-  if (relocate(fd, (unsigned char *)mbuf) < 0) 	;	/* error */
+  if (lseek(fd, (off_t)sym_bytes, SEEK_CUR) == (off_t) -1) ;	/* error */
+  if (relocate(fd, (unsigned char *)mbuf) < 0) 	;		/* error */
+  pc += (vir_bytes) rp->mp_seg[T].mem_vir << CLICK_SHIFT;
 #endif
 
   close(fd);			/* don't need exec file any more */
@@ -149,7 +167,10 @@ PUBLIC int do_exec()
   new_sp = (char *) vsp;
 
   tell_fs(EXEC, who, 0, 0);	/* allow FS to handle FD_CLOEXEC files */
-  sys_exec(who, new_sp, rmp->mp_flags & TRACED, name_buf, pc);
+
+  basename = strrchr(name_buf, '/');
+  if (basename == NULL) basename = name_buf; else basename++;
+  sys_exec(who, new_sp, rmp->mp_flags & TRACED, basename, pc);
   return(OK);
 }
 
@@ -167,11 +188,11 @@ vir_bytes *bss_bytes;		/* place to return bss size */
 phys_bytes *tot_bytes;		/* place to return total size */
 long *sym_bytes;		/* place to return symbol table size */
 vir_clicks sc;			/* stack size in clicks */
-vir_bytes *pc;
+vir_bytes *pc;			/* program entry point (initial PC) */
 {
 /* Read the header and extract the text, data, bss and total sizes from it. */
 
-  int m, ct, hdr_size;
+  int m, ct;
   vir_clicks tc, dc, s_vir, dvir;
   phys_clicks totc;
   struct exec hdr;		/* a.out header is read in here */
@@ -190,7 +211,7 @@ vir_bytes *pc;
    *	Bytes  8-11: size of text segments in bytes
    *	Bytes 12-15: size of initialized data segment in bytes
    *	Bytes 16-19: size of bss in bytes
-   *	Bytes 20-23: 0x00000000L
+   *	Bytes 20-23: program entry point
    *	Bytes 24-27: total memory allocated to program (text, data + stack)
    *	Bytes 28-31: size of symbol table in bytes
    * The longs are represented in a machine dependent order,
@@ -199,26 +220,18 @@ vir_bytes *pc;
    * are given in the header.
    */
 
-  hdr_size = sizeof(struct exec) - 4 * sizeof(long);
-  if (read(fd, (char *) &hdr, hdr_size) != hdr_size) return(ENOEXEC);
-#if (CHIP == M68000)
-  if (hdr.a_cpu == A_MAGIC0 && hdr.a_flags == A_MAGIC1)
-  {
-    /* old style 68000 executable; convert header */
+  if (read(fd, (char *) &hdr, A_MINHDR) != A_MINHDR) return(ENOEXEC);
 
-    short version;
-
-    hdr.a_flags = hdr.a_magic[1];
-    hdr.a_cpu = 0xb; /* A_M68K */
-    hdr.a_magic[0] = A_MAGIC0;
-    hdr.a_magic[1] = A_MAGIC1;
-    version = (hdr.a_unused << 8) + hdr.a_hdrlen;
-    hdr.a_hdrlen = hdr.a_version & 0xff;
-    hdr.a_unused = (hdr.a_version >> 8) & 0xff;
-    hdr.a_version = version;
-  }
+  /* Check magic number, cpu type, and flags. */
+  if (BADMAG(hdr)) return(ENOEXEC);
+#if (CHIP == INTEL && _WORD_SIZE == 2)
+  if (hdr.a_cpu != A_I8086) return(ENOEXEC);
 #endif
-  if (hdr.a_magic[0] != A_MAGIC0 || hdr.a_magic[1] != A_MAGIC1)return(ENOEXEC);
+#if (CHIP == INTEL && _WORD_SIZE == 4)
+  if (hdr.a_cpu != A_I80386) return(ENOEXEC);
+#endif
+  if ((hdr.a_flags & ~(A_NSYM | A_EXEC | A_SEP)) != 0) return(ENOEXEC);
+
   *ft = ( (hdr.a_flags & A_SEP) ? SEPARATE : 0);    /* separate I & D or not */
 
   /* Get text and data sizes. */
@@ -253,7 +266,7 @@ vir_bytes *pc;
 #endif
 
   }
-  *pc = 0x0;	/* XXX */
+  *pc = hdr.a_entry;	/* initial address to start execution */
 
   /* Check to see if segment sizes are feasible. */
   tc = ((unsigned long) *text_bytes + CLICK_SIZE - 1) >> CLICK_SHIFT;
@@ -264,7 +277,7 @@ vir_bytes *pc;
   s_vir = dvir + (totc - sc);
   m = size_ok(*ft, tc, dc, sc, dvir, s_vir);
   ct = hdr.a_hdrlen & BYTE;		/* header length */
-  if (ct > hdr_size) read(fd, (char *)&hdr, ct-hdr_size); /* skip unused hdr*/
+  if (ct > A_MINHDR) lseek(fd, (off_t) ct, SEEK_SET); /* skip unused hdr */
   return(m);
 }
 
@@ -272,14 +285,13 @@ vir_bytes *pc;
 /*===========================================================================*
  *				new_mem					     *
  *===========================================================================*/
-PRIVATE int new_mem(text_bytes, data_bytes,bss_bytes,stk_bytes,tot_bytes,bf,zs)
+PRIVATE int new_mem(sh_mp, text_bytes, data_bytes,bss_bytes,stk_bytes,tot_bytes)
+struct mproc *sh_mp;		/* text can be shared with this process */
 vir_bytes text_bytes;		/* text segment size in bytes */
 vir_bytes data_bytes;		/* size of initialized data in bytes */
 vir_bytes bss_bytes;		/* size of bss in bytes */
 vir_bytes stk_bytes;		/* size of initial stack segment in bytes */
 phys_bytes tot_bytes;		/* total memory to allocate, including gap */
-char bf[ZEROBUF_SIZE];		/* buffer to use for zeroing data segment */
-int zs;				/* true size of 'bf' */
 {
 /* Allocate new memory and release the old memory.  Change the map and report
  * the new map to the kernel.  Zero the new core image's bss, gap and stack.
@@ -292,19 +304,19 @@ int zs;				/* true size of 'bf' */
 #if (SHADOWING == 1)
   phys_clicks base, size;
 #else
-  char *rzp;
-  vir_bytes vzb;
-  phys_clicks old_clicks;
+  static char zero[1024];		/* used to zero bss */
   phys_bytes bytes, base, count, bss_offset;
 #endif
 
+  /* No need to allocate text if it can be shared. */
+  if (sh_mp != NULL) text_bytes = 0;
+
   /* Acquire the new memory.  Each of the 4 parts: text, (data+bss), gap,
    * and stack occupies an integral number of clicks, starting at click
    * boundary.  The data and bss parts are run together with no space.
    */
 
   text_clicks = ((unsigned long) text_bytes + CLICK_SIZE - 1) >> CLICK_SHIFT;
-				/* HACK */
   data_clicks = (data_bytes + bss_bytes + CLICK_SIZE - 1) >> CLICK_SHIFT;
   stack_clicks = (stk_bytes + CLICK_SIZE - 1) >> CLICK_SHIFT;
   tot_clicks = (tot_bytes + CLICK_SIZE - 1) >> CLICK_SHIFT;
@@ -321,10 +333,13 @@ int zs;				/* true size of 'bf' */
   rmp = mp;
 
 #if (SHADOWING == 0)
-  old_clicks = (phys_clicks) rmp->mp_seg[S].mem_len;
-  old_clicks += (rmp->mp_seg[S].mem_vir - rmp->mp_seg[D].mem_vir);
-  if (rmp->mp_flags & SEPARATE) old_clicks += rmp->mp_seg[T].mem_len;
-  free_mem(rmp->mp_seg[T].mem_phys, old_clicks);	/* free the memory */
+  if (find_share(rmp, rmp->mp_ino, rmp->mp_dev, rmp->mp_ctime) == NULL) {
+	/* No other process shares the text segment, so free it. */
+	free_mem(rmp->mp_seg[T].mem_phys, rmp->mp_seg[T].mem_len);
+  }
+  /* Free the data and stack segments. */
+  free_mem(rmp->mp_seg[D].mem_phys,
+      rmp->mp_seg[S].mem_vir + rmp->mp_seg[S].mem_len - rmp->mp_seg[D].mem_vir);
 #endif
 
   /* We have now passed the point of no return.  The old core image has been
@@ -332,12 +347,21 @@ int zs;				/* true size of 'bf' */
    */
   new_base = alloc_mem(text_clicks + tot_clicks);	/* new core image */
   if (new_base == NO_MEM) panic("MM hole list is inconsistent", NO_NUM);
-  rmp->mp_seg[T].mem_len = text_clicks;
-  rmp->mp_seg[T].mem_phys = new_base;
-  rmp->mp_seg[D].mem_len = data_clicks;
+
+  if (sh_mp != NULL) {
+	/* Share the text segment. */
+	rmp->mp_seg[T] = sh_mp->mp_seg[T];
+  } else {
+	rmp->mp_seg[T].mem_phys = new_base;
+	rmp->mp_seg[T].mem_vir = 0;
+	rmp->mp_seg[T].mem_len = text_clicks;
+  }
   rmp->mp_seg[D].mem_phys = new_base + text_clicks;
-  rmp->mp_seg[S].mem_len = stack_clicks;
+  rmp->mp_seg[D].mem_vir = 0;
+  rmp->mp_seg[D].mem_len = data_clicks;
   rmp->mp_seg[S].mem_phys = rmp->mp_seg[D].mem_phys + data_clicks + gap_clicks;
+  rmp->mp_seg[S].mem_vir = rmp->mp_seg[D].mem_vir + data_clicks + gap_clicks;
+  rmp->mp_seg[S].mem_len = stack_clicks;
 
 #if (CHIP == M68000)
 #if (SHADOWING == 0)
@@ -349,36 +373,33 @@ int zs;				/* true size of 'bf' */
   rmp->mp_seg[D].mem_vir = rmp->mp_seg[D].mem_phys;
   rmp->mp_seg[S].mem_vir = rmp->mp_seg[S].mem_phys;
 #endif
-#else
-  rmp->mp_seg[T].mem_vir = 0;
-  rmp->mp_seg[D].mem_vir = 0;
-  rmp->mp_seg[S].mem_vir = rmp->mp_seg[D].mem_vir + data_clicks + gap_clicks;
+#endif
+
+#if (SHADOWING == 0)
+  sys_newmap(who, rmp->mp_seg);   /* report new map to the kernel */
+
+  /* Zero the bss, gap, and stack segment. */
+  bytes = (phys_bytes)(data_clicks + gap_clicks + stack_clicks) << CLICK_SHIFT;
+  base = (phys_bytes) rmp->mp_seg[D].mem_phys << CLICK_SHIFT;
+  bss_offset = (data_bytes >> CLICK_SHIFT) << CLICK_SHIFT;
+  base += bss_offset;
+  bytes -= bss_offset;
+
+  while (bytes > 0) {
+	count = MIN(bytes, (phys_bytes) sizeof(zero));
+	if (sys_copy(MM_PROC_NR, D, (phys_bytes) zero,
+						ABS, 0, base, count) != OK) {
+		panic("new_mem can't zero", NO_NUM);
+	}
+	base += count;
+	bytes -= count;
+  }
 #endif
 
 #if (SHADOWING == 1)
   sys_fresh(who, rmp->mp_seg, (phys_clicks)(data_bytes >> CLICK_SHIFT),
 			&base, &size);
   free_mem(base, size);
-#else
-  sys_newmap(who, rmp->mp_seg);   /* report new map to the kernel */
-
-  /* Zero the bss, gap, and stack segment. Start just above text.  */
-  for (rzp = &bf[0]; rzp < &bf[zs]; rzp++) *rzp = 0;	/* clear buffer */
-  bytes = (phys_bytes)(data_clicks + gap_clicks + stack_clicks) << CLICK_SHIFT;
-  vzb = (vir_bytes) bf;
-  base = (long) rmp->mp_seg[T].mem_phys + rmp->mp_seg[T].mem_len;
-  base = base << CLICK_SHIFT;
-  bss_offset = (data_bytes >> CLICK_SHIFT) << CLICK_SHIFT;
-  base += bss_offset;
-  bytes -= bss_offset;
-
-  while (bytes > 0) {
-	count = (long) MIN(bytes, (phys_bytes) zs);
-	if (mem_copy(MM_PROC_NR, D, (long) vzb, ABS, 0, base, count) != OK)
-		panic("new_mem can't zero", NO_NUM);
-	base += count;
-	bytes -= count;
-  }
 #endif
 
   return(OK);
@@ -452,6 +473,35 @@ vir_bytes seg_bytes;		/* how big is the segment */
   }
 }
 
+
+/*===========================================================================*
+ *				find_share				     *
+ *===========================================================================*/
+PUBLIC struct mproc *find_share(mp_ign, ino, dev, ctime)
+struct mproc *mp_ign;		/* process that should not be looked at */
+ino_t ino;			/* parameters that uniquely identify a file */
+dev_t dev;
+time_t ctime;
+{
+/* Look for a process that is the file <ino, dev, ctime> in execution.  Don't
+ * accidentally "find" mp_ign, because it is the process on whose behalf this
+ * call is made.
+ */
+  struct mproc *sh_mp;
+
+  for (sh_mp = &mproc[INIT_PROC_NR]; sh_mp < &mproc[NR_PROCS]; sh_mp++) {
+	if ((sh_mp->mp_flags & (IN_USE | HANGING | SEPARATE))
+					!= (IN_USE | SEPARATE)) continue;
+	if (sh_mp == mp_ign) continue;
+	if (sh_mp->mp_ino != ino) continue;
+	if (sh_mp->mp_dev != dev) continue;
+	if (sh_mp->mp_ctime != ctime) continue;
+	return sh_mp;
+  }
+  return(NULL);
+}
+
+
 #if (SHADOWING == 1)
 /*===========================================================================*
  *				relocate				     *
