@@ -23,9 +23,3 @@ EXTERN char *res_ptr;		/* result, if pointer */
 extern _PROTOTYPE (int (*call_vec[]), (void) );	/* system call handlers */
 extern char core_name[];	/* file name where core images are produced */
 EXTERN sigset_t core_bits;	/* which signals cause core images */
-EXTERN char mm_stack[MM_STACK_BYTES];	/* MM's stack */
-
-/* DEBUG.  There is nothing to force this stack to be aligned.  char variables
- * do not require alignment on most systems.  The rounding of MM_STACK_BYTES
- * in const.h is not sufficient.
- */
