@@ -2,7 +2,9 @@
 
 #include <sys/types.h>
 #include <sys/times.h>
+#ifdef _MINIX
 #include <sys/sigcontext.h>
+#endif
 #include <sys/wait.h>
 #include <errno.h>
 #include <signal.h>
@@ -54,16 +56,14 @@ _PROTOTYPE(void test8l, (void));
 _PROTOTYPE(void func_m1, (void));
 _PROTOTYPE(void func_m2, (void));
 _PROTOTYPE(void test8m, (void));
-_PROTOTYPE(void test8n, (void));
 _PROTOTYPE(void longjerr, (void));
 _PROTOTYPE(void catch14, (int signo, int code, struct sigcontext * scp));
-_PROTOTYPE(void test8o, (void));
+_PROTOTYPE(void test8n, (void));
 _PROTOTYPE(void catch15, (int signo));
-_PROTOTYPE(void test8p, (void));
-_PROTOTYPE(void test8q, (void));
+_PROTOTYPE(void test8o, (void));
 _PROTOTYPE(void clearsigstate, (void));
 _PROTOTYPE(void quit, (void));
-_PROTOTYPE(void wait_for, (pid_t pid));
+_PROTOTYPE(void wait_for, (int pid));
 _PROTOTYPE(void e, (int n));
 
 int main(argc, argv)
@@ -100,7 +100,6 @@ char *argv[];
 	if (m & 0010000) test8m();
 	if (m & 0020000) test8n();
 	if (m & 0040000) test8o();
-	if (m & 0100000) test8p();
   }
 
   quit();
@@ -115,6 +114,7 @@ void test8a()
   sigset_t s;
 
   subtest = 1;
+  clearsigstate();
 
   /* Create an empty set and see if any bits are on. */
   if (sigemptyset(&s) != 0) e(1);
@@ -234,6 +234,7 @@ void test8b()
   struct sigaction sa, osa;
 
   subtest = 2;
+  clearsigstate();
 
   /* Construct s_ill = {SIGILL} and s_ill_pip {SIGILL | SIGPIP}, etc. */
   if (sigemptyset(&s_empty) != 0) e(1);
@@ -245,6 +246,10 @@ void test8b()
   if (sigfillset(&s_full) != 0) e(7);
   s_nokill = s_full;
   if (sigdelset(&s_nokill, SIGKILL) != 0) e(8);
+#ifndef _MINIX /* XXX - should unsupported signals be <= _NSIG? */
+  if (SIGSTOP > _NSIG) e(666);
+#endif /* _MINIX */
+  if (SIGSTOP <= _NSIG && sigdelset(&s_nokill, SIGSTOP) != 0) e(888);
 
   /* Now get most of the signals into default state.  Don't change SIGINT
   * or SIGQUIT, so this program can be killed.  SIGKILL is also special.
@@ -255,7 +260,7 @@ void test8b()
   for (i = 0; i < SIGS; i++) sigaction(i, &sa, &osa);
 
   /* The second argument may be zero.  See if it wipes out the system. */
-  for (i = 0; i < SIGS; i++) sigaction(i, (struct sigaction *) 0, &osa);
+  for (i = 0; i < SIGS; i++) sigaction(i, (struct sigaction *) NULL, &osa);
 
   /* Install a signal handler. */
   sa.sa_handler = func1;
@@ -279,7 +284,8 @@ void test8b()
   if (sigaction(SIGHUP, &sa, &osa) != 0) e(13);
   if (osa.sa_handler != func1) e(14);
   if (osa.sa_mask != s_ill) e(15);
-  if (osa.sa_flags != (SA_NODEFER | SA_NOCLDSTOP)) e(16);
+  if (osa.sa_flags != SA_NODEFER
+      && osa.sa_flags != (SA_NODEFER | SA_NOCLDSTOP)) e(16);
 
   /* Replace action once more and check what is read back. */
   sa.sa_handler = SIG_DFL;
@@ -290,7 +296,7 @@ void test8b()
   if (sigaction(SIGHUP, &sa, &osa) != 0) e(17);
   if (osa.sa_handler != func2) e(18);
   if (osa.sa_mask != s_ill_pip) e(19);
-  if (osa.sa_flags != (SA_RESETHAND | SA_NODEFER)) e(20);
+  if (osa.sa_flags != (SA_RESETHAND | SA_NODEFER | SA_COMPAT)) e(20);
 
   /* Test sigprocmask(SIG_SETMASK, ...). */
   if (sigprocmask(SIG_SETMASK, &s_full, &s1) != 0) e(18);    /* block all */
@@ -336,15 +342,15 @@ void test8b()
   if (s1 != s_nokill) e(48);
 
   /* If second arg is 0, nothing is set. */
-  if (sigprocmask(SIG_SETMASK, (sigset_t *) 0, &s1) != 0) e(49);
+  if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &s1) != 0) e(49);
   if (s1 != s_nokill) e(50);
   if (sigprocmask(SIG_SETMASK, &s_ill_pip, &s1) != 0) e(51);
   if (s1 != s_nokill) e(52);
-  if (sigprocmask(SIG_SETMASK, (sigset_t *) 0, &s1) != 0) e(53);
+  if (sigprocmask(SIG_SETMASK, (sigset_t *) NULL, &s1) != 0) e(53);
   if (s1 != s_ill_pip) e(54);
-  if (sigprocmask(SIG_BLOCK, (sigset_t *) 0, &s1) != 0) e(55);
+  if (sigprocmask(SIG_BLOCK, (sigset_t *) NULL, &s1) != 0) e(55);
   if (s1 != s_ill_pip) e(56);
-  if (sigprocmask(SIG_UNBLOCK, (sigset_t *) 0, &s1) != 0) e(57);
+  if (sigprocmask(SIG_UNBLOCK, (sigset_t *) NULL, &s1) != 0) e(57);
   if (s1 != s_ill_pip) e(58);
 
   /* Trying to block SIGKILL is not allowed, but is not an error, either. */
@@ -384,7 +390,8 @@ int signo;
 void catch2(signo)
 int signo;
 {
-  if (sigprocmask(SIG_BLOCK, (sigset_t *)NULL, (sigset_t *) & glo_vol_set) != 0) e(1);
+  if (sigprocmask(SIG_BLOCK, (sigset_t *)NULL, (sigset_t *) &glo_vol_set) != 0)
+	e(1);
 }
 
 /* Verify that signal(2), which is now built on top of sigaction(2), still
@@ -396,7 +403,6 @@ void test8c()
   sigset_t sigset_var;
 
   subtest = 3;
-
   clearsigstate();
   x = 0;
 
@@ -441,14 +447,16 @@ void test8c()
 	errct = 0;
 	while (x == 0);
 	if (x != 42) e(16);
-	if (sigismember((sigset_t *) & glo_vol_set, SIGTERM)) e(17);
+	if (sigismember((sigset_t *) &glo_vol_set, SIGTERM)) e(17);
 	if (sigprocmask(SIG_BLOCK, (sigset_t *)NULL, &sigset_var) != 0) e(18);
 	if (sigismember(&sigset_var, SIGTERM)) e(19);
 
-  /* If you have compiled signal() with the BSD behaviour, then
-   * the next line will incorrectly detect an error.
-   */
+#if 0
+/* Use this if you have compiled signal() to have the broken SYSV behaviour. */
 	if (signal(SIGTERM, catch1) != SIG_DFL) e(20);
+#else
+	if (signal(SIGTERM, catch1) != catch1) e(20);
+#endif
 	exit(errct == 0 ? 0 : 1);
       default:			/* parent */
 	sleep(1);
@@ -485,10 +493,10 @@ void test8d()
   struct sigaction act;
 
   subtest = 4;
-
   clearsigstate();
   y = 0;
   z = 0;
+
   act.sa_handler = catch3;
   act.sa_mask = 0;
   act.sa_flags = SA_NODEFER;	/* Otherwise, nested occurence of
@@ -511,13 +519,13 @@ int signo;
   sigset_t oset;
   sigset_t set;
 
-  if (sigemptyset(&set) == -1) e(1);
-  if (sigaddset(&set, SIGTERM) == -1) e(2);
-  if (sigaddset(&set, SIGHUP) == -1) e(3);
-  if (sigaddset(&set, SIGINT) == -1) e(4);
-  if (sigaddset(&set, SIGPIPE) == -10) e(5);
-  if (sigprocmask(SIG_BLOCK, (sigset_t *)NULL, &oset) != 0) e(6);
-  if (oset != set) e(7);
+  if (sigemptyset(&set) == -1) e(5001);
+  if (sigaddset(&set, SIGTERM) == -1) e(5002);
+  if (sigaddset(&set, SIGHUP) == -1) e(5003);
+  if (sigaddset(&set, SIGINT) == -1) e(5004);
+  if (sigaddset(&set, SIGPIPE) == -1) e(5005);
+  if (sigprocmask(SIG_BLOCK, (sigset_t *)NULL, &oset) != 0) e(5006);
+  if (oset != set) e(5007);
 }
 
 void test8e()
@@ -563,6 +571,7 @@ void test8f()
   pid_t pid;
 
   subtest = 6;
+  clearsigstate();
 
   switch (pid = fork()) {
       case 0:			/* child */
@@ -575,7 +584,7 @@ void test8f()
 	if (sigemptyset(&act.sa_mask) == -1) e(3);
 	act.sa_flags = 0;
 	act.sa_handler = catch5;
-	if (sigaction(SIGINT, &act, 0) == -1) e(4);
+	if (sigaction(SIGINT, &act, (struct sigaction *) NULL) == -1) e(4);
 
 	if (sigemptyset(&set) == -1) e(5);
 	r = sigsuspend(&set);
@@ -639,10 +648,10 @@ void test8g()
 	if (sigemptyset(&act.sa_mask) == -1) e(3);
 	act.sa_flags = 0;
 	act.sa_handler = sigint_handler;
-	if (sigaction(SIGINT, &act, 0) == -1) e(4);
+	if (sigaction(SIGINT, &act, (struct sigaction *) NULL) == -1) e(4);
 
 	act.sa_handler = sigpipe_handler;
-	if (sigaction(SIGPIPE, &act, 0) == -1) e(5);
+	if (sigaction(SIGPIPE, &act, (struct sigaction *) NULL) == -1) e(5);
 
 	if (sigemptyset(&set) == -1) e(6);
 	if (sigaddset(&set, SIGINT) == -1) e(7);
@@ -718,13 +727,13 @@ void test8h()
 	if (sigemptyset(&act.sa_mask) == -1) e(6);
 	act.sa_flags = 0;
 	act.sa_handler = sighup8;
-	if (sigaction(SIGHUP, &act, 0) == -1) e(7);
+	if (sigaction(SIGHUP, &act, (struct sigaction *) NULL) == -1) e(7);
 
 	act.sa_handler = sigpip8;
-	if (sigaction(SIGPIPE, &act, 0) == -1) e(8);
+	if (sigaction(SIGPIPE, &act, (struct sigaction *) NULL) == -1) e(8);
 
 	act.sa_handler = sigter8;
-	if (sigaction(SIGTERM, &act, 0) == -1) e(9);
+	if (sigaction(SIGTERM, &act, (struct sigaction *) NULL) == -1) e(9);
 
 	if (sigemptyset(&set) == -1) e(10);
 	if (sigaddset(&set, SIGHUP) == -1) e(11);
@@ -763,8 +772,8 @@ void test8i()
   sigset_t set;
   struct sigaction act;
 
-  clearsigstate();
   subtest = 9;
+  clearsigstate();
   y = 0;
   z = 0;
 
@@ -772,10 +781,10 @@ void test8i()
   act.sa_flags = 0;
 
   act.sa_handler = sighup9;
-  if (sigaction(SIGHUP, &act, 0) == -1) e(2);
+  if (sigaction(SIGHUP, &act, (struct sigaction *) NULL) == -1) e(2);
 
   act.sa_handler = sigter9;
-  if (sigaction(SIGTERM, &act, 0) == -1) e(3);
+  if (sigaction(SIGTERM, &act, (struct sigaction *) NULL) == -1) e(3);
 
   if (sigemptyset(&set) == -1) e(4);
   if (sigaddset(&set, SIGTERM) == -1) e(5);
@@ -799,7 +808,7 @@ void test8i()
 *
 * Install signal handlers for SIGALRM and SIGINT.
 *
-* Set an alarm for 6 second, then sleep for 7.
+* Set an alarm for 6 seconds, then sleep for 7.
 *
 * The SIGALRM should interrupt the sleep, but the SIGINT
 * should remain pending.
@@ -822,9 +831,8 @@ void test8j()
   sigset_t set, set2;
   struct sigaction act;
 
-  clearsigstate();
-
   subtest = 10;
+  clearsigstate();
   y = 0;
   z = 0;
 
@@ -832,10 +840,10 @@ void test8j()
   act.sa_flags = 0;
 
   act.sa_handler = sighup10;
-  if (sigaction(SIGHUP, &act, 0) == -1) e(2);
+  if (sigaction(SIGHUP, &act, (struct sigaction *) NULL) == -1) e(2);
 
   act.sa_handler = sigalrm_handler10;
-  if (sigaction(SIGALRM, &act, 0) == -1) e(3);
+  if (sigaction(SIGALRM, &act, (struct sigaction *) NULL) == -1) e(3);
 
   if (sigemptyset(&set) == -1) e(4);
   if (sigaddset(&set, SIGHUP) == -1) e(5);
@@ -858,38 +866,11 @@ void test8j()
 
 void test8k()
 {
+  subtest = 11;
 }
 void test8l()
 {
-}
-
-/*--------------------------------------------------------------------------*/
-
-/* Basic test for _setjmp/_longjmp. */
-
-jmp_buf glo_jb;
-
-void func_m1()
-{
-  func_m2();
-}
-
-void func_m2()
-{
-  z++;
-  _longjmp(glo_jb, 1);
-}
-
-void test8m()
-{
-  subtest = 13;
-  z = 0;
-
-  if (_setjmp(glo_jb)) {
-	if (z != 1) e(1);
-	return;
-  }
-  func_m1();
+  subtest = 12;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -898,13 +879,14 @@ void test8m()
 * signal mask is properly restored.
 */
 
-void test8n()
+void test8m()
 {
   jmp_buf jb;
   sigset_t ss;
 
   subtest = 13;
   clearsigstate();
+
   ss = 0x32;
   if (sigprocmask(SIG_SETMASK, &ss, (sigset_t *)NULL) == -1) e(1);
   if (setjmp(jb)) {
@@ -946,9 +928,10 @@ struct sigcontext *scp;
   e(2);
 }
 
-void test8o()
+void test8n()
 {
   struct sigaction act;
+  typedef _PROTOTYPE( void (*sighandler_t), (int sig) );
 
   subtest = 14;
   clearsigstate();
@@ -957,7 +940,7 @@ void test8o()
 
   act.sa_flags = 0;
   act.sa_mask = 0;
-  act.sa_handler = (sighandler_t) catch14;
+  act.sa_handler = (sighandler_t) catch14;	/* fudge */
   if (sigaction(SIGSEGV, &act, (struct sigaction *) NULL) == -1) e(3);
   if (kill(getpid(), SIGSEGV) == -1) e(4);
 
@@ -971,6 +954,7 @@ void test8o()
  *
  * Catch a signal.  Longjmp out of signal handler.
  */
+jmp_buf glo_jb;
 
 void catch15(signo)
 int signo;
@@ -981,7 +965,7 @@ int signo;
 
 }
 
-void test8p()
+void test8o()
 {
   struct sigaction act;
   int k;
@@ -1003,20 +987,14 @@ void test8p()
   if (kill(getpid(), SIGALRM) == -1) e(5);
 }
 
-void test8q()
-{
-}
-
-
-
 void clearsigstate()
 {
   int i;
   sigset_t sigset_var;
 
   /* Clear the signal state. */
-  for (i = 1; i < _NSIG; i++) signal(i, SIG_IGN);
-  for (i = 1; i < _NSIG; i++) signal(i, SIG_DFL);
+  for (i = 1; i <= _NSIG; i++) signal(i, SIG_IGN);
+  for (i = 1; i <= _NSIG; i++) signal(i, SIG_DFL);
   sigfillset(&sigset_var);
   sigprocmask(SIG_UNBLOCK, &sigset_var, (sigset_t *)NULL);
 }
